@@ -21,7 +21,7 @@
  * 2005-12-09  Peter Niederlag  <peter@niederlag.de>
  *	- Fixed bug #1976: PHP5 type-conversion of string 'true' and boolean
  *
- * $Id: class.gzip_encode.php 2663 2007-11-05 09:22:23Z ingmars $
+ * $Id: class.gzip_encode.php 4516 2008-12-01 16:25:00Z dmitry $
  *
  * @author	Sandy McArthur, Jr. <leknor@leknor.com>
  */
@@ -339,13 +339,15 @@ class gzip_encode {
 	 * @return	float		Returns the current load average
 	 */
 	function linux_loadavg() {
-		$buffer = "0 0 0";
-		$f = fopen("/proc/loadavg","rb");
-		if (!feof($f)) {
-		    $buffer = fgets($f, 1024);
+		$buffer = '0 0 0';
+		$f = @fopen('/proc/loadavg', 'rb');
+		if ($f) {
+			if (!feof($f)) {
+				$buffer = fgets($f, 1024);
+			}
+			fclose($f);
 		}
-		fclose($f);
-		$load = explode(" ",$buffer);
+		$load = explode(' ', $buffer);
 		return max((float)$load[0], (float)$load[1], (float)$load[2]);
 	}
 
