@@ -27,7 +27,7 @@
 /**
  * Contains class for display of backend log
  *
- * $Id: class.t3lib_bedisplaylog.php 3503 2008-04-01 19:50:18Z stucki $
+ * $Id: class.t3lib_bedisplaylog.php 4978 2009-02-10 07:50:24Z stucki $
  * Revised for TYPO3 3.6 July/2003 by Kasper Skaarhoj
  * XHTML compliant
  *
@@ -141,7 +141,7 @@ class t3lib_BEDisplayLog {
 			$this->lastUserLabel=$code.'_'.$workspace;
 			$label = $this->be_user_Array[$code]['username'];
 			$ws = $this->wsArray[$workspace];
-			return ($label ? $label : '['.$code.']').'@'.($ws?$ws:$workspace);
+			return ($label ? htmlspecialchars($label) : '['.$code.']').'@'.($ws?$ws:$workspace);
 		} else return '.';
 	}
 
@@ -163,13 +163,13 @@ class t3lib_BEDisplayLog {
 	 * Get action label for log listing
 	 *
 	 * @param	string		Key for the action label in locallang
-	 * @return	string		If labe is different from last action label then the label is returned, otherwise "."
+	 * @return	string		If label is different from last action label then the label is returned, otherwise "."
 	 */
 	function getActionLabel($code)	{
 		if ($this->lastActionLabel!=$code)	{
 			$this->lastActionLabel=$code;
 			$label=$GLOBALS['LANG']->getLL('action_'.$code);
-			return $label ? $label : '['.$code.']';
+			return $label ? htmlspecialchars($label) : '['.$code.']';
 		} else return '.';
 	}
 
@@ -192,7 +192,9 @@ class t3lib_BEDisplayLog {
 				} else {
 					list($label) = explode(',',$text);
 				}
-				if ($label)	{$text=$label;}
+				if ($label)	{
+					$text=$label;
+				}
 				$text = sprintf($text, htmlspecialchars($data[0]),htmlspecialchars($data[1]),htmlspecialchars($data[2]),htmlspecialchars($data[3]),htmlspecialchars($data[4]));
 			} else {
 				$text = str_replace('%s','',$text);
