@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2006 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2008 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -27,7 +27,7 @@
 /**
  * extending class to class t3lib_basicFileFunctions
  *
- * $Id: class.t3lib_extfilefunc.php 1657 2006-07-26 17:54:15Z mundaun $
+ * $Id: class.t3lib_extfilefunc.php 4737 2009-01-16 22:15:53Z steffenk $
  * Revised for TYPO3 3.6 May/2004 by Kasper Skaarhoj
  *
  * @author	Kasper Skaarhoj <kasperYYYY@typo3.com>
@@ -94,7 +94,7 @@
  * You are allowed to copy/move folders between spaces (web/ftp) IF the destination has it's f_ext[]['allow'] set to '*'!
  *
  * Advice:
- * You should always exclude php-files from the webspace. This will keep people from uploading, copy/moving and renaming files to the php3/php-extension.
+ * You should always exclude php-files from the webspace. This will keep people from uploading, copy/moving and renaming files to become executable php scripts.
  * You should never mount a ftp_space 'below' the webspace so that it reaches into the webspace. This is because if somebody unzips a zip-file in the ftp-space so that it reaches out into the webspace this will be a violation of the safety
  * For example this is a bad idea: you have an ftp-space that is '/www/' and a web-space that is '/www/htdocs/'
  *
@@ -139,6 +139,8 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 		// Internal, dynamic:
 	var $internalUploadMap = array();	// Will contain map between upload ID and the final filename
 
+	var $lastError = '';
+
 
 
 
@@ -215,6 +217,19 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 
 		if (is_array($this->fileCmdMap))	{
 
+				// Check if there were uploads expected, but no one made
+			if ($this->fileCmdMap['upload'])	{
+				$uploads = $this->fileCmdMap['upload'];
+				foreach ($uploads as $arr)	{
+					if (!$_FILES['upload_'.$arr['data']]['name'])	{
+						unset($this->fileCmdMap['upload'][$arr['data']]);
+					}
+				}
+				if (count($this->fileCmdMap['upload']) == 0) {
+					$this->writelog(1,1,108,'No file was uploaded!','');
+				}
+			}
+
 				// Traverse each set of actions
 			foreach($this->fileCmdMap as $action => $actionData)	{
 
@@ -359,6 +374,7 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 		if (is_object($GLOBALS['BE_USER']))	{
 			$GLOBALS['BE_USER']->writelog($type,$action,$error,$details_nr,$details,$data);
 		}
+		$this->lastError = vsprintf($details, $data);
 	}
 
 
@@ -418,12 +434,8 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 					$theFile = $this->is_directory($theFile);
 					if ($theFile)	{
 						if ($this->checkPathAgainstMounts($theFile))	{	// I choose not to append '/' to $theFile here as this will prevent us from deleting mounts!! (which makes sense to me...)
-							if ($this->actionPerms['deleteFolderRecursively'] && !$this->dont_use_exec_commands)	{
-									// No way to do this under windows
-								$cmd = 'rm -Rf "'.$theFile.'"';
-								exec($cmd);		// This is a quite critical command...
-								clearstatcache();
-								if (!@file_exists($theFile))	{
+							if ($this->actionPerms['deleteFolderRecursively'])	{
+								if (t3lib_div::rmdir($theFile,true))	{
 									$this->writelog(4,0,2,'Directory "%s" deleted recursively!',Array($theFile));
 									return TRUE;
 								} else $this->writelog(4,2,119,'Directory "%s" WAS NOT deleted recursively! Write-permission problem?',Array($theFile));
@@ -491,7 +503,7 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 									return $theDestFile;
 								} else $this->writelog(2,2,109,'File "%s" WAS NOT copied to "%s"! Write-permission problem?',Array($theFile,$theDestFile));
 							} else	$this->writelog(2,1,110,'Target or destination was not within your mountpoints! T="%s", D="%s"',Array($theFile,$theDestFile));
-						} else $this->writelog(2,1,111,'Fileextension "%s" is not allowed in "%s"!',Array($fI['fileext'],$theDest.'/'));
+						} else $this->writelog(2,1,111,'Extension of file name "%s" is not allowed in "%s"!',Array($fI['file'],$theDest.'/'));
 					} else $this->writelog(2,1,112,'File "%s" already exists!',Array($theDestFile));
 				} else $this->writelog(2,1,113,'File "%s" exceeds the size-limit of %s bytes',Array($theFile,$this->maxCopyFileSize*1024));
 			} else $this->writelog(2,1,114,'You are not allowed to copy files','');
@@ -581,7 +593,7 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 									return $theDestFile;
 								} else $this->writelog(3,2,109,'File "%s" WAS NOT moved to "%s"! Write-permission problem?',Array($theFile,$theDestFile));
 							} else $this->writelog(3,1,110,'Target or destination was not within your mountpoints! T="%s", D="%s"',Array($theFile,$theDestFile));
-						} else $this->writelog(3,1,111,'Fileextension "%s" is not allowed in "%s"!',Array($fI['fileext'],$theDest.'/'));
+						} else $this->writelog(3,1,111,'Extension of file name "%s" is not allowed in "%s"!',Array($fI['file'],$theDest.'/'));
 					} else $this->writelog(3,1,112,'File "%s" already exists!',Array($theDestFile));
 				} else $this->writelog(3,1,113,'File "%s" exceeds the size-limit of %s bytes',Array($theFile,$this->maxMoveFileSize*1024));
 			} else $this->writelog(3,1,114,'You are not allowed to move files','');
@@ -656,7 +668,7 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 												$this->writelog(5,0,1,'File renamed from "%s" to "%s"',Array($fileInfo['file'],$theNewName));
 												return $theRenameName;
 											} else $this->writelog(5,1,100,'File "%s" was not renamed! Write-permission problem in "%s"?',Array($theTarget,$fileInfo['path']));
-										} else $this->writelog(5,1,101,'Fileextension "%s" was not allowed!',Array($fI['fileext']));
+										} else $this->writelog(5,1,101,'Extension of file name "%s" was not allowed!',Array($fI['file']));
 									} else $this->writelog(5,1,102,'You are not allowed to rename files!','');
 								} elseif ($type=='dir')	{
 									if ($this->actionPerms['renameFolder'])	{
@@ -684,7 +696,7 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 		if (!$this->isInit) return FALSE;
 
 		$theFolder = $this->cleanFileName($cmds['data']);
-		if ($theFolder)	{
+		if (isset($theFolder))	{
 			if ($this->checkFileNameLen($theFolder))	{
 				$theTarget = $this->is_directory($cmds['target']);	// Check the target dir
 				if ($theTarget)	{
@@ -732,7 +744,7 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 											return $theNewFile;
 										} else $this->writelog(8,1,100,'File "%s" was not created! Write-permission problem in "%s"?',Array($fI['file'], $theTarget));
 									} else $this->writelog(8,1,107,'Fileextension "%s" is not a textfile format! (%s)',Array($fI['fileext'], $extList));
-								} else $this->writelog(8,1,106,'Fileextension "%s" was not allowed!',Array($fI['fileext']));
+								} else $this->writelog(8,1,106,'Extension of file name "%s" was not allowed!',Array($fI['file']));
 							} else $this->writelog(8,1,101,'File "%s" existed already!',Array($theNewFile));
 						} else $this->writelog(8,1,102,'Destination path "%s" was not within your mountpoints!',Array($theTarget.'/'));
 					} else $this->writelog(8,1,103,'You are not allowed to create files!','');
@@ -767,7 +779,7 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 								return TRUE;
 							} else $this->writelog(9,1,100,'File "%s" was not saved! Write-permission problem in "%s"?',Array($theTarget,$fileInfo['path']));
 						} else $this->writelog(9,1,102,'Fileextension "%s" is not a textfile format! (%s)',Array($fI['fileext'], $extList));
-					} else $this->writelog(9,1,103,'Fileextension "%s" was not allowed!',Array($fI['fileext']));
+					} else $this->writelog(9,1,103,'Extension of file name "%s" was not allowed!',Array($fI['file']));
 				} else $this->writelog(9,1,104,'You are not allowed to edit files!','');
 			} else $this->writelog(9,1,121,'Destination path "%s" was not within your mountpoints!',Array($fileInfo['path']));
 		} else $this->writelog(9,2,123,'Target "%s" was not a file!',Array($theTarget));
@@ -803,12 +815,12 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 										return $theNewFile;
 									} else $this->writelog(1,1,100,'Uploaded file could not be moved! Write-permission problem in "%s"?',Array($theTarget.'/'));
 								} else $this->writelog(1,1,101,'No unique filename available in "%s"!',Array($theTarget.'/'));
-							} else $this->writelog(1,1,102,'Fileextension "%s" is not allowed in "%s"!',Array($fI['fileext'],$theTarget.'/'));
+							} else $this->writelog(1,1,102,'Extension of file name "%s" is not allowed in "%s"!',Array($fI['file'], $theTarget.'/'));
 						} else $this->writelog(1,1,103,'Destination path "%s" was not within your mountpoints!',Array($theTarget.'/'));
 					} else $this->writelog(1,1,104,'The uploaded file exceeds the size-limit of %s bytes',Array($this->maxUploadFileSize*1024));
 				} else $this->writelog(1,1,105,'You are not allowed to upload files!','');
-			} else $this->writelog(1,2,106,'The uploaded file did not exist!','');
-		} else $this->writelog(1,2,106,'No file was uploaded!','');
+			} else $this->writelog(1,2,106,'The upload has failed, no uploaded file found!','');
+		} else $this->writelog(1,2,108,'No file was uploaded!','');
 	}
 
 	/**
