@@ -27,7 +27,7 @@
 /**
  * Class for generating front end for building queries
  *
- * $Id: class.t3lib_querygenerator.php 3489 2008-03-31 13:13:04Z ohader $
+ * $Id: class.t3lib_querygenerator.php 4881 2009-01-28 13:58:15Z benni $
  *
  * @author	Christian Jul Jensen <christian@typo3.com>
  * @author	Kasper Skaarhoj <kasperYYYY@typo3.com>
@@ -1482,7 +1482,9 @@ class t3lib_queryGenerator	{
 				$qString .= ' AND pid IN ('.$webMountPageTree.')';
 			}
 		}
-		$fieldlist = $this->extFieldLists['queryFields'].',pid,deleted';
+		$fieldlist = $this->extFieldLists['queryFields'] .
+			',pid' .
+			($GLOBALS['TCA'][$this->table]['ctrl']['delete'] ? ',' . $GLOBALS['TCA'][$this->table]['ctrl']['delete'] : '');
 		if (!$GLOBALS['SOBE']->MOD_SETTINGS['show_deleted'])	{
 			$qString .= t3lib_BEfunc::deleteClause($this->table);
 		}
