@@ -27,7 +27,7 @@
 /**
  * Class for TYPO3 backend user authentication in the TSFE frontend
  *
- * $Id: class.t3lib_tsfebeuserauth.php 3763 2008-06-05 11:51:40Z flyguide $
+ * $Id: class.t3lib_tsfebeuserauth.php 4931 2009-02-03 16:55:46Z jsegars $
  * Revised for TYPO3 3.6 July/2003 by Kasper Skaarhoj
  * XHTML compliant
  *
@@ -193,7 +193,7 @@ class t3lib_tsfeBeUserAuth extends t3lib_beUserAuth {
 	ADMIN PANEL
 -->
 <a name="TSFE_ADMIN"></a>
-<form name="TSFE_ADMIN_PANEL_FORM" action="'.htmlspecialchars(t3lib_div::getIndpEnv('SCRIPT_NAME')).'#TSFE_ADMIN" method="get" style="margin:0;" onsubmit="document.forms.TSFE_ADMIN_PANEL_FORM[\'TSFE_ADMIN_PANEL[DUMMY]\'].value=Math.random().toString().substring(2,8)">'.
+<form name="TSFE_ADMIN_PANEL_FORM" action="'.htmlspecialchars(t3lib_div::getIndpEnv('TYPO3_REQUEST_SCRIPT')).'#TSFE_ADMIN" method="get" style="margin:0;" onsubmit="document.forms.TSFE_ADMIN_PANEL_FORM[\'TSFE_ADMIN_PANEL[DUMMY]\'].value=Math.random().toString().substring(2,8)">'.
 $query.'
 	<table border="0" cellpadding="0" cellspacing="0" class="typo3-adminPanel" style="background-color:#f6f2e6; border: 1px solid black; z-index:0; position:absolute;" summary="">'.
 		$header.
