@@ -37,7 +37,7 @@
  *
  *
  *
- * $Id: alt_main.php 3570 2008-04-16 06:26:27Z flyguide $
+ * $Id: alt_main.php 4978 2009-02-10 07:50:24Z stucki $
  * Revised for TYPO3 3.6 2/2003 by Kasper Skaarhoj
  * XHTML Compliant (almost)
  *
@@ -185,7 +185,7 @@ class SC_alt_main {
 	function typoSetup()	{	//
 		this.PATH_typo3 = "'.$pt3.'";
 		this.PATH_typo3_enc = "'.rawurlencode($pt3).'";
-		this.username = "'.$BE_USER->user['username'].'";
+		this.username = "'.htmlspecialchars($BE_USER->user['username']).'";
 		this.uniqueID = "'.t3lib_div::shortMD5(uniqid('')).'";
 		this.navFrameWidth = 0;
 	}
