@@ -266,9 +266,11 @@ class TYPO3backend {
 					<li><div id="logout-button" class="toolbar-item no-separator">'.$this->moduleMenu->renderLogoutButton().'</div></li>';
 
 		foreach($this->toolbarItems as $toolbarItem) {
-			$additionalAttributes = $toolbarItem->getAdditionalAttributes();
-
-			$toolbar .= '<li'.$additionalAttributes.'>'.$toolbarItem->render().'</li>';
+			$menu = $toolbarItem->render();
+			if ($menu) {
+				$additionalAttributes = $toolbarItem->getAdditionalAttributes();   
+				$toolbar .= '<li' . $additionalAttributes . '>' .$menu. '</li>';
+			}
 		}
 
 		return $toolbar.'</ul>';
@@ -379,7 +381,7 @@ class TYPO3backend {
 	function typoSetup()	{	//
 		this.PATH_typo3 = "'.$pathTYPO3.'";
 		this.PATH_typo3_enc = "'.rawurlencode($pathTYPO3).'";
-		this.username = "'.$GLOBALS['BE_USER']->user['username'].'";
+		this.username = "'.htmlspecialchars($GLOBALS['BE_USER']->user['username']).'";
 		this.uniqueID = "'.t3lib_div::shortMD5(uniqid('')).'";
 		this.navFrameWidth = 0;
 	}
