@@ -28,7 +28,7 @@
  * Libraries for pagegen.php
  * The script "pagegen.php" is included by "index_ts.php" when a page is not cached but needs to be rendered.
  *
- * $Id: class.tslib_pagegen.php 3759 2008-06-05 10:01:32Z ohader $
+ * $Id: class.tslib_pagegen.php 4586 2008-12-21 16:54:06Z dmitry $
  * Revised for TYPO3 3.6 June/2003 by Kasper Skaarhoj
  * XHTML compliant
  *
@@ -486,9 +486,10 @@ See <a href="http://wiki.typo3.org/index.php/TYPO3_3.8.1" target="_blank">wiki.t
 			$favIcon = t3lib_div::getIndpEnv('TYPO3_SITE_URL') . $GLOBALS['TSFE']->tmpl->getFileName($GLOBALS['TSFE']->pSetup['shortcutIcon']);
 			$iconMimeType = '';
 			if (function_exists('finfo_open')) {
-				$finfo = finfo_open(FILEINFO_MIME);
-				$iconMimeType = ' type="'.finfo_file($finfo,$favIcon).'"';
-				finfo_close($finfo);
+				if (($finfo = @finfo_open(FILEINFO_MIME))) {
+					$iconMimeType = ' type="' . finfo_file($finfo, $favIcon) . '"';
+					finfo_close($finfo);
+				}
 			}
 
 			$GLOBALS['TSFE']->content.= '
@@ -648,7 +649,7 @@ See <a href="http://wiki.typo3.org/index.php/TYPO3_3.8.1" target="_blank">wiki.t
 						if (!$type)	$type = 'text/javascript';
 
 						$GLOBALS['TSFE']->content.='
-	<script src="'.htmlspecialchars($ss).'" type="'.htmlspecialchars($type).'"></script>';
+	<script src="' . htmlspecialchars($GLOBALS['TSFE']->absRefPrefix . $ss) . '" type="' . htmlspecialchars($type) . '"></script>';
 					}
 				}
 			}
