@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 2001-2005 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 2001-2008 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -27,7 +27,7 @@
 /**
  * Index search frontend
  *
- * $Id: class.tx_indexedsearch.php 1421 2006-04-10 09:27:15Z mundaun $
+ * $Id: class.tx_indexedsearch.php 3439 2008-03-16 19:16:51Z flyguide $
  *
  * Creates a searchform for indexed search. Indexing must be enabled
  * for this to make sense.
@@ -154,7 +154,20 @@ class tx_indexedsearch extends tslib_pibase {
 	var $resultSections = array();		// Page tree sections for search result.
 	var $external_parsers = array();	// External parser objects
 	var $iconFileNameCache = array();	// Storage of icons....
-	var $lexerObj;				// Lexer object
+
+	/**
+	 * Lexer object
+	 *
+	 * @var tx_indexedsearch_lexer
+	 */
+	var $lexerObj;
+
+	/**
+	 * Indexer object
+	 *
+	 * @var tx_indexedsearch_indexer
+	 */
+	var $indexerObj;
 	var $templateCode;			// Will hold the content of $conf['templateFile']
 	var $hiddenFieldList = 'ext, type, defOp, media, order, group, lang, desc, results';
 
@@ -304,6 +317,10 @@ class tx_indexedsearch extends tslib_pibase {
 			}
 		}
 
+			// Should we use join_pages instead of long lists of uids?
+		if ($this->conf['search.']['skipExtendToSubpagesChecking'])	{
+			$this->join_pages = 1;
+		}
 
 			// Add media to search in:
 		if (strlen(trim($this->conf['search.']['mediaList'])))	{
@@ -332,12 +349,14 @@ class tx_indexedsearch extends tslib_pibase {
 		if ($this->conf['show.']['L1sections'])	{
 			$firstLevelMenu = $this->getMenu($this->wholeSiteIdList);
 			while(list($kk,$mR) = each($firstLevelMenu))	{
+					// @TODO: RFC #7370: doktype 2&5 are deprecated since TYPO3 4.2-beta1
 				if ($mR['doktype']!=5)	{
 					$this->optValues['sections']['rl1_'.$mR['uid']] = trim($this->pi_getLL('opt_RL1').' '.$mR['title']);
 					if ($this->conf['show.']['L2sections'])	{
 						$secondLevelMenu = $this->getMenu($mR['uid']);
 						while(list($kk2,$mR2) = each($secondLevelMenu))	{
-							if ($mR['doktype']!=5)	{
+								// @TODO: RFC #7370: doktype 2&5 are deprecated since TYPO3 4.2-beta1
+							if ($mR2['doktype']!=5)	{
 								$this->optValues['sections']['rl2_'.$mR2['uid']] = trim($this->pi_getLL('opt_RL2').' '.$mR2['title']);
 							} else unset($secondLevelMenu[$kk2]);
 						}
@@ -564,8 +583,10 @@ class tx_indexedsearch extends tslib_pibase {
 			$c = 0;	// Result pointer: Counts up the position in the current search-result
 			$grouping_phashes = array();	// Used to filter out duplicates.
 			$grouping_chashes = array();	// Used to filter out duplicates BASED ON cHash.
-			$firstRow = Array();	// Will hold the first row in result - used to calculate relative hit-ratings.
-			$resultRows = Array();	// Will hold the results rows for display.
+			$firstRow = array();	// Will hold the first row in result - used to calculate relative hit-ratings.
+			$resultRows = array();	// Will hold the results rows for display.
+
+			$exactCount = $this->conf['search.']['exactCount'];	// Continue counting and checking of results even if we are sure they are not displayed in this request. This will slow down your page rendering, but it allows precise search result counters.
 
 				// Now, traverse result and put the rows to be displayed into an array
 				// Each row should contain the fields from 'ISEC.*, IP.*' combined + artificial fields "show_resume" (boolean) and "result_number" (counter)
@@ -593,8 +614,8 @@ class tx_indexedsearch extends tslib_pibase {
 						if ($c > $pointer * $this->piVars['results'])	{
 							$row['result_number'] = $c;
 							$resultRows[] = $row;
-								// This may lead to a problem: If the result check is not stopped here, the search will take longer. However the result counter will not filter out grouped cHashes/pHashes that were not processed yet.
-							if (($c+1) > ($pointer+1)*$this->piVars['results'])	break;
+								// This may lead to a problem: If the result check is not stopped here, the search will take longer. However the result counter will not filter out grouped cHashes/pHashes that were not processed yet. You can change this behavior using the "search.exactCount" property (see above).
+							if (!$exactCount && (($c+1) > ($pointer+1)*$this->piVars['results']))	{ break; }
 						}
 					} else {
 						$count--;	// Skip this row if the user cannot view it (missing permission)
@@ -682,7 +703,7 @@ class tx_indexedsearch extends tslib_pibase {
 			// Print a message telling which words we searched for, and in which sections etc.
 		$what = $this->tellUsWhatIsSeachedFor($sWArr).
 				(substr($this->piVars['sections'],0,2)=='rl'?' '.$this->pi_getLL('inSection','',1).' "'.substr($this->getPathFromPageId(substr($this->piVars['sections'],4)),1).'"':'');
-		$what = '<div'.$this->pi_classParam('whatis').'><p>'.$what.'</p></div>';
+		$what = '<div'.$this->pi_classParam('whatis').'>'.$this->cObj->stdWrap($what, $this->conf['whatis_stdWrap.']).'</div>';
 		$content = $what.$content;
 
 			// Return content:
@@ -820,23 +841,23 @@ class tx_indexedsearch extends tslib_pibase {
 
 				// Perform search for word:
 			switch($theType)	{
-				case '1':
+				case '1':	// Part of word
 					$wSel = "IW.baseword LIKE '%".$GLOBALS['TYPO3_DB']->quoteStr($sWord, 'index_words')."%'";
 					$res = $this->execPHashListQuery($wSel,' AND is_stopword=0');
 				break;
-				case '2':
+				case '2':	// First part of word
 					$wSel = "IW.baseword LIKE '".$GLOBALS['TYPO3_DB']->quoteStr($sWord, 'index_words')."%'";
 					$res = $this->execPHashListQuery($wSel,' AND is_stopword=0');
 				break;
-				case '3':
+				case '3':	// Last part of word
 					$wSel = "IW.baseword LIKE '%".$GLOBALS['TYPO3_DB']->quoteStr($sWord, 'index_words')."'";
 					$res = $this->execPHashListQuery($wSel,' AND is_stopword=0');
 				break;
-				case '10':
+				case '10':	// Sounds like
 					$wSel = 'IW.metaphone = '.$this->indexerObj->metaphone($sWord);
 					$res = $this->execPHashListQuery($wSel,' AND is_stopword=0');
 				break;
-				case '20':
+				case '20':	// Sentence
 					$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery(
 								'ISEC.phash',
 								'index_section ISEC, index_fulltext IFT',
@@ -849,7 +870,7 @@ class tx_indexedsearch extends tslib_pibase {
 
 					if ($this->piVars['type']==20)	$this->piVars['order'] = 'mtime';		// If there is a fulltext search for a sentence there is a likeliness that sorting cannot be done by the rankings from the rel-table (because no relations will exist for the sentence in the word-table). So therefore mtime is used instaed. It is not required, but otherwise some hits may be left out.
 				break;
-				default:
+				default:	// Distinct word
 					$wSel = 'IW.wid = '.$hash = $this->indexerObj->md5inthash($sWord);
 					$res = $this->execPHashListQuery($wSel,' AND is_stopword=0');
 				break;
@@ -1148,7 +1169,7 @@ class tx_indexedsearch extends tslib_pibase {
 							AND ISEC.phash = IR.phash
 							AND IP.phash = IR.phash
 							AND	'.$page_where,
-						'IP.phash,ISEC.phash,ISEC.phash_t3,ISEC.rl0,ISEC.rl1,ISEC.rl2 ,ISEC.page_id,ISEC.uniqid,IP.phash_grouping,IP.data_filename ,IP.data_page_id ,IP.data_page_reg1,IP.data_page_type,IP.data_page_mp,IP.gr_list,IP.item_type,IP.item_title,IP.item_description,IP.item_mtime,IP.tstamp,IP.item_size,IP.contentHash,IP.crdate,IP.parsetime,IP.sys_language_uid,IP.item_crdate,IP.cHashParams,IP.externalUrl,IP.recordUid,IP.freeIndexUid',
+						'IP.phash,ISEC.phash,ISEC.phash_t3,ISEC.rl0,ISEC.rl1,ISEC.rl2 ,ISEC.page_id,ISEC.uniqid,IP.phash_grouping,IP.data_filename ,IP.data_page_id ,IP.data_page_reg1,IP.data_page_type,IP.data_page_mp,IP.gr_list,IP.item_type,IP.item_title,IP.item_description,IP.item_mtime,IP.tstamp,IP.item_size,IP.contentHash,IP.crdate,IP.parsetime,IP.sys_language_uid,IP.item_crdate,IP.cHashParams,IP.externalUrl,IP.recordUid,IP.freeIndexUid,IP.freeIndexSetId',
 						$orderBy
 					);
 		} else {	// Otherwise, if sorting are done with the pages table or other fields, there is no need for joining with the rel/word tables:
@@ -1175,7 +1196,7 @@ class tx_indexedsearch extends tslib_pibase {
 							$freeIndexUidClause.'
 							AND IP.phash = ISEC.phash
 							AND '.$page_where,
-						'IP.phash,ISEC.phash,ISEC.phash_t3,ISEC.rl0,ISEC.rl1,ISEC.rl2 ,ISEC.page_id,ISEC.uniqid,IP.phash_grouping,IP.data_filename ,IP.data_page_id ,IP.data_page_reg1,IP.data_page_type,IP.data_page_mp,IP.gr_list,IP.item_type,IP.item_title,IP.item_description,IP.item_mtime,IP.tstamp,IP.item_size,IP.contentHash,IP.crdate,IP.parsetime,IP.sys_language_uid,IP.item_crdate,IP.cHashParams,IP.externalUrl,IP.recordUid,IP.freeIndexUid',
+						'IP.phash,ISEC.phash,ISEC.phash_t3,ISEC.rl0,ISEC.rl1,ISEC.rl2 ,ISEC.page_id,ISEC.uniqid,IP.phash_grouping,IP.data_filename ,IP.data_page_id ,IP.data_page_reg1,IP.data_page_type,IP.data_page_mp,IP.gr_list,IP.item_type,IP.item_title,IP.item_description,IP.item_mtime,IP.tstamp,IP.item_size,IP.contentHash,IP.crdate,IP.parsetime,IP.sys_language_uid,IP.item_crdate,IP.cHashParams,IP.externalUrl,IP.recordUid,IP.freeIndexUid,IP.freeIndexSetId',
 						$orderBy
 					);
 		}
@@ -1307,7 +1328,7 @@ class tx_indexedsearch extends tslib_pibase {
 			// Multilangual text
 		$substituteArray = array('searchFor', 'extResume', 'atATime', 'orderBy', 'fromSection', 'searchIn', 'match', 'style', 'freeIndexUid');
 		foreach ($substituteArray as $marker)	{
-			$markerArray['###FORM_'.strtoupper($marker).'###'] = $this->pi_getLL('form_'.$marker,'',1);
+			$markerArray['###FORM_'.t3lib_div::strtoupper($marker).'###'] = $this->pi_getLL('form_'.$marker,'',1);
 		}
 
 		$markerArray['###FORM_SUBMIT###'] = $this->pi_getLL('submit_button_label','',1);
@@ -1324,7 +1345,7 @@ class tx_indexedsearch extends tslib_pibase {
 			$html = $this->cObj->substituteSubpart($html, '###ADDITONAL_KEYWORD###', '');
 		}
 
-		$markerArray['###ACTION_URL###'] = $this->pi_getPageLink($GLOBALS['TSFE']->id, $GLOBALS['TSFE']->sPre);
+		$markerArray['###ACTION_URL###'] = htmlspecialchars($this->pi_getPageLink($GLOBALS['TSFE']->id, $GLOBALS['TSFE']->sPre));
 
 		$hiddenFieldCode = $this->cObj->getSubpart($this->templateCode, '###HIDDEN_FIELDS###');
 		$hiddenFieldCode = preg_replace('/^\n\t(.+)/ms', '$1', $hiddenFieldCode);		// Remove first newline and tab (cosmetical issue)
@@ -1333,7 +1354,7 @@ class tx_indexedsearch extends tslib_pibase {
 		foreach (t3lib_div::trimExplode(',',$this->hiddenFieldList) as $fieldName)	{
 			$hiddenFieldMarkerArray = array();
 			$hiddenFieldMarkerArray['###HIDDEN_FIELDNAME###'] = $this->prefixId.'['.$fieldName.']';
-			$hiddenFieldMarkerArray['###HIDDEN_VALUE###'] = (string)$this->piVars[$fieldName];
+			$hiddenFieldMarkerArray['###HIDDEN_VALUE###'] = htmlspecialchars((string)$this->piVars[$fieldName]);
 
 			$hiddenFieldArr[$fieldName] = $this->cObj->substituteMarkerArrayCached($hiddenFieldCode, $hiddenFieldMarkerArray, array(), array());
 		}
@@ -1575,7 +1596,7 @@ class tx_indexedsearch extends tslib_pibase {
 
 			if (is_array($tmplContent))	{
 				foreach ($tmplContent AS $k => $v)	{
-					$markerArray['###'.strtoupper($k).'###'] = $v;
+					$markerArray['###'.t3lib_div::strtoupper($k).'###'] = $v;
 				}
 			}
 
@@ -1958,7 +1979,7 @@ class tx_indexedsearch extends tslib_pibase {
 	/**
 	 * Marks up the search words from $this->sWarr in the $str with a color.
 	 *
-	 * @param	string		text in which to find and mark up search words. This text is assumed to be UTF-8 like the search words internally is.
+	 * @param	string		Text in which to find and mark up search words. This text is assumed to be UTF-8 like the search words internally is.
 	 * @return	string		Processed content.
 	 */
 	function markupSWpartsOfString($str)	{
@@ -1970,13 +1991,13 @@ class tx_indexedsearch extends tslib_pibase {
 
 			// Prepare search words for regex:
 		foreach ($this->sWArr as $d)	{
-			$swForReg[] = quotemeta($d['sword']);
+			$swForReg[] = preg_quote($d['sword'],'/');
 		}
 		$regExString = '('.implode('|',$swForReg).')';
 
 			// Split and combine:
 		$parts = preg_split('/'.$regExString.'/i', ' '.$str.' ', 20000, PREG_SPLIT_DELIM_CAPTURE);
-#debug($parts,$regExString);
+// debug($parts,$regExString);
 			// Constants:
 		$summaryMax = 300;
 		$postPreLgd = 60;
@@ -2066,9 +2087,12 @@ class tx_indexedsearch extends tslib_pibase {
 	 * @return	array		Modified template array
 	 */
 	function makeInfo($row,$tmplArray)	{
+		$dateFormat = $GLOBALS['TYPO3_CONF_VARS']['SYS']['ddmmyy'];
+		$timeFormat = $GLOBALS['TYPO3_CONF_VARS']['SYS']['hhmm'];
+
 		$tmplArray['size'] = t3lib_div::formatSize($row['item_size']);
-		$tmplArray['created'] = date('d-m-y',$row['item_crdate']);
-		$tmplArray['modified'] = date('d-m-y H:i',$row['item_mtime']);
+		$tmplArray['created'] = date($dateFormat, $row['item_crdate']);
+		$tmplArray['modified'] = date($dateFormat.' '.$timeFormat, $row['item_mtime']);
 
 		$pathId = $row['data_page_id']?$row['data_page_id']:$row['page_id'];
 		$pathMP = $row['data_page_id']?$row['data_page_mp']:'';
@@ -2310,6 +2334,7 @@ class tx_indexedsearch extends tslib_pibase {
 			while($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res))	{
 				$output[$row['uid']] = $GLOBALS['TSFE']->sys_page->getPageOverlay($row);
 			}
+			$GLOBALS['TYPO3_DB']->sql_free_result($res);
 			return $output;
 		} else {
 			return $GLOBALS['TSFE']->sys_page->getMenu($id);
@@ -2360,4 +2385,4 @@ class tx_indexedsearch extends tslib_pibase {
 if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/indexed_search/pi/class.tx_indexedsearch.php'])	{
 	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/indexed_search/pi/class.tx_indexedsearch.php']);
 }
-?>
\ No newline at end of file
+?>
