@@ -32,7 +32,7 @@
  * @author	Philipp Borgmann <philipp.borgmann@gmx.de>
  * @author	Stanislas Rolland <typo3(arobas)sjbr.ca>
  *
- * $Id: class.tx_rtehtmlarea_base.php 3478 2008-03-28 20:08:23Z stan $  *
+ * $Id: class.tx_rtehtmlarea_base.php 4670 2009-01-08 21:10:33Z stan $  *
  */
 
 require_once(PATH_t3lib.'class.t3lib_rteapi.php');
@@ -106,7 +106,7 @@ class tx_rtehtmlarea_base extends t3lib_rteapi {
 		'redo'			=> 'Redo',
 		);
 	
-	var $pluginList = 'ContextMenu';
+	var $pluginList;
 	var $pluginButton = array();
 	var $pluginLabel = array();
 	
@@ -151,7 +151,7 @@ class tx_rtehtmlarea_base extends t3lib_rteapi {
 	var $toolbarOrderArray = array();
 	protected $pluginEnabledArray = array();		// Array of plugin id's enabled in the current RTE editing area
 	protected $pluginEnabledCumulativeArray = array();	// Cumulative array of plugin id's enabled so far in any of the RTE editing areas of the form
-	protected $registeredPlugins = array();			// Array of registered plugins indexd by their plugin Id's
+	protected $registeredPlugins = array();			// Array of registered plugins indexed by their plugin Id's
 	
 	/**
 	 * Returns true if the RTE is available. Here you check if the browser requirements are met.
@@ -335,18 +335,16 @@ class tx_rtehtmlarea_base extends t3lib_rteapi {
 				// htmlArea plugins list
 			$this->pluginEnabledArray = t3lib_div::trimExplode(',', $this->pluginList, 1);
 			$this->enableRegisteredPlugins();
-			$hidePlugins = array();
+
 			if ($this->client['BROWSER'] == 'msie') {
 				$this->thisConfig['keepButtonGroupTogether'] = 0;
 			}
 			if ($this->client['BROWSER'] == 'opera') {
-				$hidePlugins[] = 'ContextMenu';
 				$this->thisConfig['keepButtonGroupTogether'] = 0;
 			}
 			if ($this->client['BROWSER'] == 'gecko' && $this->client['VERSION'] == '1.3')  {
 				$this->thisConfig['keepButtonGroupTogether'] = 0;
 			}
-			$this->pluginEnabledArray = array_diff($this->pluginEnabledArray, $hidePlugins);
 
 				// Toolbar
 			$this->setToolbar();
@@ -381,54 +379,25 @@ class tx_rtehtmlarea_base extends t3lib_rteapi {
 			 * LOAD CSS AND JAVASCRIPT
 			 * =======================================
 			 */
-
-				// Preloading the pageStyle
-			$filename = trim($this->thisConfig['contentCSS']) ? trim($this->thisConfig['contentCSS']) : 'EXT:' . $this->ID . '/res/contentcss/default.css';
-			$this->TCEform->additionalCode_pre['loadCSS'] = '
-		<link rel="alternate stylesheet" type="text/css" href="' . $this->getFullFileName($filename) . '" title="HTMLArea RTE Content CSS" />';
-
-				// Loading the editor skin
-			$skinFilename = trim($this->thisConfig['skin']) ? trim($this->thisConfig['skin']) : 'EXT:' . $this->ID . '/htmlarea/skins/default/htmlarea.css';
-			if($this->client['BROWSER'] == 'gecko' && $this->client['VERSION'] == '1.3' && substr($skinFilename,0,4) == 'EXT:')  {
-				$skinFilename = 'EXT:' . $this->ID . '/htmlarea/skins/default/htmlarea.css';
-			}
-			if (substr($skinFilename,0,4) == 'EXT:')      {       // extension
-				list($extKey,$local) = explode('/',substr($skinFilename,4),2);
-				$skinFilename='';
-				if (strcmp($extKey,'') &&  t3lib_extMgm::isLoaded($extKey) && strcmp($local,'')) {
-					$skinFilename = $this->httpTypo3Path . t3lib_extMgm::siteRelPath($extKey) . $local;
-					$skinDir = $this->siteURL . t3lib_extMgm::siteRelPath($extKey) . dirname($local);
-				}
-			} elseif (substr($skinFilename,0,1) != '/') {
-				$skinDir = $this->siteURL.dirname($skinFilename);
-				$skinFilename = $this->siteURL . $skinFilename;
+				// Preloading the pageStyle and including RTE skin stylesheets
+			if (is_object($GLOBALS['TSFE'])) {
+					// Frontend editing
+				$GLOBALS['TSFE']->additionalHeaderData['rtehtmlarea-contentCSS'] = $this->getPageStyle();
+				$GLOBALS['TSFE']->additionalHeaderData['rtehtmlarea-skin'] = $this->getSkin();
 			} else {
-				$skinDir = substr($this->siteURL,0,-1) . dirname($skinFilename);
-			}
-			$this->editorCSS = $skinFilename;
-			$this->editedContentCSS = $skinDir . '/htmlarea-edited-content.css';
-			$this->TCEform->additionalCode_pre['loadCSS'] .= '
-		<link rel="alternate stylesheet" type="text/css" href="' . $this->editedContentCSS . '" />';
-			
-				// Main skin
-			$this->TCEform->additionalCode_pre['loadCSS'] .= '
-		<link rel="stylesheet" type="text/css" href="' . $this->editorCSS . '" />';
-			
-				// Additional icons from registered plugins
-			foreach ($this->pluginEnabledCumulativeArray[$this->TCEform->RTEcounter] as $pluginId) {
-				if (is_object($this->registeredPlugins[$pluginId])) {
-					$pathToSkin = $this->registeredPlugins[$pluginId]->getPathToSkin();
-					if ($pathToSkin) {
-						$this->TCEform->additionalCode_pre['loadCSS'] .= '
-		<link rel="stylesheet" type="text/css" href="' . $this->httpTypo3Path . t3lib_extMgm::siteRelPath($this->registeredPlugins[$pluginId]->getExtensionKey()) . $pathToSkin . '" />';
-					}
+					// If it was not known that an RTE-enabled field would be created when the page was first created, the css would not have been added to head
+				if (is_object($this->TCEform->inline) && $this->TCEform->inline->isAjaxCall) {
+					$this->TCEform->additionalCode_pre['rtehtmlarea-contentCSS'] = $this->getPageStyle();
+					$this->TCEform->additionalCode_pre['rtehtmlarea-skin'] = $this->getSkin();
+				} else {
+					$GLOBALS['SOBE']->doc->additionalHeaderData['rtehtmlarea-contentCSS'] = $this->getPageStyle();
+					$GLOBALS['SOBE']->doc->additionalHeaderData['rtehtmlarea-skin'] = $this->getSkin();
 				}
 			}
-			
 				// Loading JavaScript files and code
 			if ($this->TCEform->RTEcounter == 1) {
-				$this->TCEform->additionalCode_pre['loadJSfiles'] = $this->loadJSfiles($this->TCEform->RTEcounter);
-				$this->TCEform->additionalJS_pre['loadJScode'] = $this->loadJScode($this->TCEform->RTEcounter);
+				$this->TCEform->additionalCode_pre['rtehtmlarea-loadJSfiles'] = $this->loadJSfiles($this->TCEform->RTEcounter);
+				$this->TCEform->additionalJS_pre['rtehtmlarea-loadJScode'] = $this->loadJScode($this->TCEform->RTEcounter);
 			}
 
 			/* =======================================
@@ -447,6 +416,7 @@ class tx_rtehtmlarea_base extends t3lib_rteapi {
 			}
 				// Register RTE windows
 			$this->TCEform->RTEwindows[] = $PA['itemFormElName'];
+			$textAreaId = htmlspecialchars($PA['itemFormElName']);
 
 				// Check if wizard_rte called this for fullscreen edtition; if so, change the size of the RTE to fullscreen using JS
 			if (basename(PATH_thisScript) == 'wizard_rte.php') {
@@ -459,21 +429,22 @@ class tx_rtehtmlarea_base extends t3lib_rteapi {
 				$editorWrapWidth = '100%';
 				$editorWrapHeight = '100%';
 				$this->RTEdivStyle = 'position:relative; left:0px; top:0px; height:100%; width:100%; border: 1px solid black; padding: 2px 0px 2px 2px;';
-				$this->TCEform->additionalJS_post[] = $this->setRTEsizeByJS('RTEarea'.$this->TCEform->RTEcounter, $height, $width);
+				$this->TCEform->additionalJS_post[] = $this->setRTEsizeByJS('RTEarea' . $textAreaId, $height, $width);
 			}
 
 				// Register RTE in JS:
-			$this->TCEform->additionalJS_post[] = $this->registerRTEinJS($this->TCEform->RTEcounter, $table, $row['uid'], $field);
+			$this->TCEform->additionalJS_post[] = $this->registerRTEinJS($this->TCEform->RTEcounter, $table, $row['uid'], $field, $textAreaId);
 
 				// Set the save option for the RTE:
-			$this->TCEform->additionalJS_submit[] = $this->setSaveRTE($this->TCEform->RTEcounter, $this->TCEform->formName, htmlspecialchars($PA['itemFormElName']));
+			$this->TCEform->additionalJS_submit[] = $this->setSaveRTE($this->TCEform->RTEcounter, $this->TCEform->formName, $textAreaId);
+			$this->TCEform->additionalJS_delete[] = $this->setDeleteRTE($this->TCEform->RTEcounter, $this->TCEform->formName, $textAreaId);
 
 				// Draw the textarea
 			$visibility = 'hidden';
 			$item = $this->triggerField($PA['itemFormElName']).'
-				<div id="pleasewait' . $this->TCEform->RTEcounter . '" class="pleasewait" style="display: none;" >' . $LANG->getLL('Please wait') . '</div>
-				<div id="editorWrap' . $this->TCEform->RTEcounter . '" class="editorWrap" style="width:' . $editorWrapWidth . '; height:' . $editorWrapHeight . ';">
-				<textarea id="RTEarea'.$this->TCEform->RTEcounter.'" name="'.htmlspecialchars($PA['itemFormElName']).'" style="'.t3lib_div::deHSCentities(htmlspecialchars($this->RTEdivStyle)).'">'.t3lib_div::formatForTextarea($value).'</textarea>
+				<div id="pleasewait' . $textAreaId . '" class="pleasewait" style="display: block;" >' . $LANG->getLL('Please wait') . '</div>
+				<div id="editorWrap' . $textAreaId . '" class="editorWrap" style="visibility: hidden; width:' . $editorWrapWidth . '; height:' . $editorWrapHeight . ';">
+				<textarea id="RTEarea' . $textAreaId . '" name="'.htmlspecialchars($PA['itemFormElName']).'" style="'.t3lib_div::deHSCentities(htmlspecialchars($this->RTEdivStyle)).'">'.t3lib_div::formatForTextarea($value).'</textarea>
 				</div>' . ($TYPO3_CONF_VARS['EXTCONF'][$this->ID]['enableDebugMode'] ? '<div id="HTMLAreaLog"></div>' : '') . '
 				';
 		}
@@ -481,7 +452,63 @@ class tx_rtehtmlarea_base extends t3lib_rteapi {
 			// Return form item:
 		return $item;
 	}
-	
+
+	/**
+	 * Get links to content style sheet(s)
+	 *
+	 * @return	string	link tag referenceing the content stylesheet
+	 */
+	protected function getPageStyle() {
+			// Get stylesheet file name from Page TSConfig if any
+		$filename = trim($this->thisConfig['contentCSS']) ? trim($this->thisConfig['contentCSS']) : 'EXT:' . $this->ID . '/res/contentcss/default.css';
+		return '<link rel="alternate stylesheet" type="text/css" href="' . $this->getFullFileName($filename) . '" title="HTMLArea RTE Content CSS" />';
+	}
+
+	/**
+	 * Get links to skin style sheet(s)
+	 *
+	 * @return	string	link tags referenceing the skin stylesheets
+	 */
+	protected function getSkin() {
+			// Get skin file name from Page TSConfig if any
+		$skinFilename = trim($this->thisConfig['skin']) ? trim($this->thisConfig['skin']) : 'EXT:' . $this->ID . '/htmlarea/skins/default/htmlarea.css';
+		if($this->client['BROWSER'] == 'gecko' && $this->client['VERSION'] == '1.3' && substr($skinFilename,0,4) == 'EXT:')  {
+			$skinFilename = 'EXT:' . $this->ID . '/htmlarea/skins/default/htmlarea.css';
+		}
+			// Skin provided by some extension
+		if (substr($skinFilename,0,4) == 'EXT:') {
+			list($extKey,$local) = explode('/',substr($skinFilename,4),2);
+			$skinFilename='';
+			if (strcmp($extKey,'') &&  t3lib_extMgm::isLoaded($extKey) && strcmp($local,'')) {
+				$skinFilename = $this->httpTypo3Path . t3lib_extMgm::siteRelPath($extKey) . $local;
+				$skinDir = $this->siteURL . t3lib_extMgm::siteRelPath($extKey) . dirname($local);
+			}
+		} elseif (substr($skinFilename,0,1) != '/') {
+			$skinDir = $this->siteURL.dirname($skinFilename);
+			$skinFilename = $this->siteURL . $skinFilename;
+		} else {
+			$skinDir = substr($this->siteURL,0,-1) . dirname($skinFilename);
+		}
+		$this->editorCSS = $skinFilename;
+			// Editing area style sheet
+		$this->editedContentCSS = $skinDir . '/htmlarea-edited-content.css';
+		$skin = '<link rel="alternate stylesheet" type="text/css" href="' . $this->editedContentCSS . '" title="HTMLArea RTE Editing Area CSS" />';
+			// Main skin
+		$skin .= '
+<link rel="stylesheet" type="text/css" href="' . $this->editorCSS . '" title="HTMLArea RTE Skin" />';
+			// Additional icons from registered plugins
+		foreach ($this->pluginEnabledCumulativeArray[$this->TCEform->RTEcounter] as $pluginId) {
+			if (is_object($this->registeredPlugins[$pluginId])) {
+				$pathToSkin = $this->registeredPlugins[$pluginId]->getPathToSkin();
+				if ($pathToSkin) {
+					$skin .= '
+<link rel="stylesheet" type="text/css" href="' . $this->httpTypo3Path . t3lib_extMgm::siteRelPath($this->registeredPlugins[$pluginId]->getExtensionKey()) . $pathToSkin . '" />';
+				}
+			}
+		}
+		return $skin;
+	}
+
 	/**
 	 * Add registered plugins to the array of enabled plugins
 	 *
@@ -517,11 +544,11 @@ class tx_rtehtmlarea_base extends t3lib_rteapi {
 			// Process overrides
 		$hidePlugins = array();
 		foreach ($this->registeredPlugins as $pluginId => $plugin) {
-			if (!$this->pluginButton[$pluginId]) {
+			if ($plugin->addsButtons() && !$this->pluginButton[$pluginId]) {
 				$hidePlugins[] = $pluginId;
 			}
 		}
-		$this->pluginEnabledArray = array_diff($this->pluginEnabledArray, $hidePlugins);
+		$this->pluginEnabledArray = array_unique(array_diff($this->pluginEnabledArray, $hidePlugins));
 	}
 	
 	/**
@@ -659,37 +686,50 @@ class tx_rtehtmlarea_base extends t3lib_rteapi {
 	 * Disable some plugins
 	 *
 	 */
-
 	function setPlugins() {
-		global $TYPO3_CONF_VARS;
 		
-		$hideButtons = array();
-		
-			// Disabling the plugins if their buttons are not in the toolbar
+			// Disabling a plugin that adds buttons if none of its buttons is in the toolbar
 		$hidePlugins = array();
 		foreach ($this->pluginButton as $pluginId => $buttonList) {
-			$showPlugin = false;
-			$buttonArray = t3lib_div::trimExplode(',', $buttonList, 1);
-			foreach ($buttonArray as $button) {
-				if (in_array($button, $this->toolbar)) {
-					$showPlugin = true;
+			if ($this->registeredPlugins[$pluginId]->addsButtons()) {
+				$showPlugin = false;
+				$buttonArray = t3lib_div::trimExplode(',', $buttonList, 1);
+				foreach ($buttonArray as $button) {
+					if (in_array($button, $this->toolbar)) {
+						$showPlugin = true;
+					}
+				}
+				if (!$showPlugin) {
+					$hidePlugins[] = $pluginId;
 				}
 			}
-			if (!$showPlugin) {
-				$hidePlugins[] = $pluginId;
-			}
 		}
-		if ($this->thisConfig['disableContextMenu'] || $this->thisConfig['disableRightClick']) $hidePlugins[] = 'ContextMenu';
 		$this->pluginEnabledArray = array_diff($this->pluginEnabledArray, $hidePlugins);
 		
 			// Hiding labels of disabled plugins
+		$hideLabels = array();
 		foreach ($this->pluginLabel as $pluginId => $label) {
 			if (!$this->isPluginEnabled($pluginId)) {
-				$hideButtons[] = $label;
+				$hideLabels[] = $label;
 			}
 		}
-		$this->toolbar = array_diff($this->toolbar, $hideButtons);
-		
+		$this->toolbar = array_diff($this->toolbar, $hideLabels);
+
+			// Adding plugins declared as prerequisites by enabled plugins
+		$requiredPlugins = array();
+		foreach ($this->registeredPlugins as $pluginId => $plugin) {
+			if ($this->isPluginEnabled($pluginId)) {
+				$requiredPlugins = array_merge($requiredPlugins, t3lib_div::trimExplode(',', $plugin->getRequiredPlugins(), 1));
+			}
+		}
+		$requiredPlugins = array_unique($requiredPlugins);
+		foreach ($requiredPlugins as $pluginId) {
+			if (is_object($this->registeredPlugins[$pluginId]) && !$this->isPluginEnabled($pluginId)) {
+				$this->pluginEnabledArray[] = $pluginId;
+			}
+		}
+		$this->pluginEnabledArray = array_unique($this->pluginEnabledArray);
+
 			// Completing the toolbar converion array for htmlArea
 		foreach ($this->registeredPlugins as $pluginId => $plugin) {
 			if ($this->isPluginEnabled($pluginId)) {
@@ -733,33 +773,52 @@ class tx_rtehtmlarea_base extends t3lib_rteapi {
 	function loadJSfiles($RTEcounter) {
 		global $TYPO3_CONF_VARS;
 		
+		$loadPluginCode = '
+						HTMLArea_plugins = new Array();';
+		foreach ($this->pluginEnabledCumulativeArray[$RTEcounter] as $pluginId) {
+			$extensionKey = is_object($this->registeredPlugins[$pluginId]) ? $this->registeredPlugins[$pluginId]->getExtensionKey() : $this->ID;
+			$loadPluginCode .= '
+						HTMLArea_plugins.push("' . $this->writeTemporaryFile('EXT:' . $extensionKey . '/htmlarea/plugins/' . $pluginId . '/' . strtolower(preg_replace('/([a-z])([A-Z])([a-z])/', "$1".'-'."$2"."$3", $pluginId)) . '.js', $pluginId) . '");';
+		}
+			// Avoid re-initialization on AJax call when RTEarea object was already initialized
 		$loadJavascriptCode = '
 		<script type="text/javascript">
 		/*<![CDATA[*/
-			i=1;
-			while (document.getElementById("pleasewait" + i)) {
-				document.getElementById("pleasewait" + i).style.display = "block";
-				document.getElementById("editorWrap" + i).style.visibility = "hidden";
-				i++;
-			};
-			RTEarea = new Array();
-			RTEarea[0] = new Object();
-			RTEarea[0]["version"] = "' . $TYPO3_CONF_VARS['EXTCONF'][$this->ID]['version'] . '";'
-			. (($this->client['BROWSER'] == 'msie') ? ('
-			RTEarea[0]["htmlarea-ie"] = "' . $this->writeTemporaryFile('EXT:' . $this->ID . '/htmlarea/htmlarea-ie.js', "htmlarea-ie") . '";')
-			: ('
-			RTEarea[0]["htmlarea-gecko"] = "' . $this->writeTemporaryFile('EXT:' . $this->ID . '/htmlarea/htmlarea-gecko.js', "htmlarea-gecko") . '";')) . '
-			_editor_url = "' . $this->extHttpPath . 'htmlarea";
-			_editor_lang = "' . $this->language . '";
-			_editor_CSS = "' . $this->editorCSS . '";
-			_editor_skin = "' . dirname($this->editorCSS) . '";
-			_editor_edited_content_CSS = "' .  $this->editedContentCSS  . '";
-			_typo3_host_url = "' . $this->hostURL . '";
-			_editor_debug_mode = ' . ($TYPO3_CONF_VARS['EXTCONF'][$this->ID]['enableDebugMode'] ? 'true' : 'false') . ';
-			_editor_compressed_scripts = ' . ($TYPO3_CONF_VARS['EXTCONF'][$this->ID]['enableCompressedScripts'] ? 'true' : 'false') . ';'
-			. (($this->client['BROWSER'] == 'gecko') ? ('
-			_editor_mozAllowClipboard_url = "' . ($TYPO3_CONF_VARS['EXTCONF'][$this->ID]['mozAllowClipboardURL'] ? $TYPO3_CONF_VARS['EXTCONF'][$this->ID]['mozAllowClipboardURL'] : '') . '";')
-			: '') . '
+			if (typeof(RTEarea) == "undefined") {
+				RTEarea = new Object();
+				RTEarea.init = function() {
+					if (typeof(HTMLArea) == "undefined") {
+						window.setTimeout("RTEarea.init();", 40);
+					} else {'
+						. $loadPluginCode . '
+						HTMLArea.init();
+					}
+				};
+				RTEarea.initEditor = function(editorNumber) {
+					if (typeof(HTMLArea) == "undefined") {
+						window.setTimeout("RTEarea.initEditor(\'" + editorNumber + "\');", 40);
+					} else {
+						HTMLArea.initEditor(editorNumber);
+					}
+				};
+				RTEarea[0] = new Object();
+				RTEarea[0].version = "' . $TYPO3_CONF_VARS['EXTCONF'][$this->ID]['version'] . '";'
+				. (($this->client['BROWSER'] == 'msie') ? ('
+				RTEarea[0]["htmlarea-ie"] = "' . $this->writeTemporaryFile('EXT:' . $this->ID . '/htmlarea/htmlarea-ie.js', "htmlarea-ie") . '";')
+				: ('
+				RTEarea[0]["htmlarea-gecko"] = "' . $this->writeTemporaryFile('EXT:' . $this->ID . '/htmlarea/htmlarea-gecko.js', "htmlarea-gecko") . '";')) . '
+				_editor_url = "' . $this->extHttpPath . 'htmlarea";
+				_editor_lang = "' . $this->language . '";
+				_editor_CSS = "' . $this->editorCSS . '";
+				_editor_skin = "' . dirname($this->editorCSS) . '";
+				_editor_edited_content_CSS = "' .  $this->editedContentCSS  . '";
+				_typo3_host_url = "' . $this->hostURL . '";
+				_editor_debug_mode = ' . ($TYPO3_CONF_VARS['EXTCONF'][$this->ID]['enableDebugMode'] ? 'true' : 'false') . ';
+				_editor_compressed_scripts = ' . ($TYPO3_CONF_VARS['EXTCONF'][$this->ID]['enableCompressedScripts'] ? 'true' : 'false') . ';'
+				. (($this->client['BROWSER'] == 'gecko') ? ('
+				_editor_mozAllowClipboard_url = "' . ($TYPO3_CONF_VARS['EXTCONF'][$this->ID]['mozAllowClipboardURL'] ? $TYPO3_CONF_VARS['EXTCONF'][$this->ID]['mozAllowClipboardURL'] : '') . '";')
+				: '') . '
+			}
 		/*]]>*/
 		</script>';
 		$loadJavascriptCode .= '
@@ -778,19 +837,9 @@ class tx_rtehtmlarea_base extends t3lib_rteapi {
 	 */
 	 
 	function loadJScode($RTEcounter) {
-		global $TYPO3_CONF_VARS;
-		
-		$loadPluginCode = '';
-		foreach ($this->pluginEnabledCumulativeArray[$RTEcounter] as $pluginId) {
-			$extensionKey = is_object($this->registeredPlugins[$pluginId]) ? $this->registeredPlugins[$pluginId]->getExtensionKey() : $this->ID;
-			$loadPluginCode .= '
-			HTMLArea.loadPlugin("' . $pluginId . '", true, "' . $this->writeTemporaryFile('EXT:' . $extensionKey . '/htmlarea/plugins/' . $pluginId . '/' . strtolower(preg_replace('/([a-z])([A-Z])([a-z])/', "$1".'-'."$2"."$3", $pluginId)) . '.js', $pluginId) . '");';
-		}
 		return (!$this->is_FE() ? '' : '
-		' . '/*<![CDATA[*/') . ($this->is_FE() ? '' : '
-			RTEarea[0]["RTEtsConfigParams"] = "&RTEtsConfigParams=' . rawurlencode($this->RTEtsConfigParams()) . '";')
-			. $loadPluginCode .  '
-			HTMLArea.init();' . (!$this->is_FE() ? '' : '
+		' . '/*<![CDATA[*/') . '
+			RTEarea.init();' . (!$this->is_FE() ? '' : '
 		/*]]>*/
 		');
 	}
@@ -805,63 +854,74 @@ class tx_rtehtmlarea_base extends t3lib_rteapi {
 	 *
 	 * @return	string		the Javascript code for configuring the RTE
 	 */
-	function registerRTEinJS($RTEcounter, $table='', $uid='', $field='') {
+	function registerRTEinJS($RTEcounter, $table='', $uid='', $field='', $textAreaId = '') {
 		global $TYPO3_CONF_VARS;
 		
 		$configureRTEInJavascriptString = (!$this->is_FE() ? '' : '
 			' . '/*<![CDATA[*/') . '
-			RTEarea['.$RTEcounter.'] = new Object();
-			RTEarea['.$RTEcounter.'].RTEtsConfigParams = "&RTEtsConfigParams=' . rawurlencode($this->RTEtsConfigParams()) . '";
-			RTEarea['.$RTEcounter.'].number = '.$RTEcounter.';
-			RTEarea['.$RTEcounter.'].id = "RTEarea'.$RTEcounter.'";
-			RTEarea['.$RTEcounter.'].enableWordClean = ' . (trim($this->thisConfig['enableWordClean'])?'true':'false') . ';
-			RTEarea['.$RTEcounter.']["htmlRemoveComments"] = ' . (trim($this->thisConfig['removeComments'])?'true':'false') . ';
-			RTEarea['.$RTEcounter.'].disableEnterParagraphs = ' . (trim($this->thisConfig['disableEnterParagraphs'])?'true':'false') . ';
-			RTEarea['.$RTEcounter.'].disableObjectResizing = ' . (trim($this->thisConfig['disableObjectResizing'])?'true':'false') . ';
-			RTEarea['.$RTEcounter.']["removeTrailingBR"] = ' . (trim($this->thisConfig['removeTrailingBR'])?'true':'false') . ';
-			RTEarea['.$RTEcounter.']["useCSS"] = ' . (trim($this->thisConfig['useCSS'])?'true':'false') . ';
-			RTEarea['.$RTEcounter.']["keepButtonGroupTogether"] = ' . (trim($this->thisConfig['keepButtonGroupTogether'])?'true':'false') . ';
-			RTEarea['.$RTEcounter.']["disablePCexamples"] = ' . (trim($this->thisConfig['disablePCexamples'])?'true':'false') . ';
-			RTEarea['.$RTEcounter.']["statusBar"] = ' . (trim($this->thisConfig['showStatusBar'])?'true':'false') . ';
-			RTEarea['.$RTEcounter.']["showTagFreeClasses"] = ' . (trim($this->thisConfig['showTagFreeClasses'])?'true':'false') . ';
-			RTEarea['.$RTEcounter.']["useHTTPS"] = ' . ((trim(stristr($this->siteURL, 'https')) || $this->thisConfig['forceHTTPS'])?'true':'false') . ';
-			RTEarea['.$RTEcounter.']["enableMozillaExtension"] = ' . (($this->client['BROWSER'] == 'gecko' && $TYPO3_CONF_VARS['EXTCONF'][$this->ID]['enableMozillaExtension'])?'true':'false') . ';
-			RTEarea['.$RTEcounter.']["tceformsNested"] = ' . (is_object($this->TCEform) && method_exists($this->TCEform, 'getDynNestedStack') ? $this->TCEform->getDynNestedStack(true) : '[]') . ';';
+			if (typeof(configureEditorInstance) == "undefined") {
+				configureEditorInstance = new Object();
+			}
+			configureEditorInstance["' . $textAreaId . '"] = function() {
+				if (typeof(RTEarea) == "undefined" || typeof(HTMLArea) == "undefined") {
+					window.setTimeout("configureEditorInstance[\'' . $textAreaId . '\']();", 40);
+				} else {
+			editornumber = "' . $textAreaId . '";
+			RTEarea[editornumber] = new Object();
+			RTEarea[editornumber].RTEtsConfigParams = "&RTEtsConfigParams=' . rawurlencode($this->RTEtsConfigParams()) . '";'
+			. ($RTEcounter ? 'RTEarea[0].RTEtsConfigParams = "&RTEtsConfigParams=' . rawurlencode($this->RTEtsConfigParams()) . '";' : '') . '
+			RTEarea[editornumber].number = editornumber;
+			RTEarea[editornumber].deleted = false;
+			RTEarea[editornumber].textAreaId = "' . $textAreaId . '";
+			RTEarea[editornumber].id = "RTEarea" + editornumber;
+			RTEarea[editornumber].enableWordClean = ' . (trim($this->thisConfig['enableWordClean'])?'true':'false') . ';
+			RTEarea[editornumber]["htmlRemoveComments"] = ' . (trim($this->thisConfig['removeComments'])?'true':'false') . ';
+			RTEarea[editornumber].disableEnterParagraphs = ' . (trim($this->thisConfig['disableEnterParagraphs'])?'true':'false') . ';
+			RTEarea[editornumber].disableObjectResizing = ' . (trim($this->thisConfig['disableObjectResizing'])?'true':'false') . ';
+			RTEarea[editornumber]["removeTrailingBR"] = ' . (trim($this->thisConfig['removeTrailingBR'])?'true':'false') . ';
+			RTEarea[editornumber]["useCSS"] = ' . (trim($this->thisConfig['useCSS'])?'true':'false') . ';
+			RTEarea[editornumber]["keepButtonGroupTogether"] = ' . (trim($this->thisConfig['keepButtonGroupTogether'])?'true':'false') . ';
+			RTEarea[editornumber]["disablePCexamples"] = ' . (trim($this->thisConfig['disablePCexamples'])?'true':'false') . ';
+			RTEarea[editornumber]["statusBar"] = ' . (trim($this->thisConfig['showStatusBar'])?'true':'false') . ';
+			RTEarea[editornumber]["showTagFreeClasses"] = ' . (trim($this->thisConfig['showTagFreeClasses'])?'true':'false') . ';
+			RTEarea[editornumber]["useHTTPS"] = ' . ((trim(stristr($this->siteURL, 'https')) || $this->thisConfig['forceHTTPS'])?'true':'false') . ';
+			RTEarea[editornumber]["enableMozillaExtension"] = ' . (($this->client['BROWSER'] == 'gecko' && $TYPO3_CONF_VARS['EXTCONF'][$this->ID]['enableMozillaExtension'])?'true':'false') . ';
+			RTEarea[editornumber].tceformsNested = ' . (is_object($this->TCEform) && method_exists($this->TCEform, 'getDynNestedStack') ? $this->TCEform->getDynNestedStack(true) : '[]') . ';';
 
 			// The following properties apply only to the backend
 		if (!$this->is_FE()) {
 			$configureRTEInJavascriptString .= '
-			RTEarea['.$RTEcounter.'].sys_language_content = "' . $this->contentLanguageUid . '";
-			RTEarea['.$RTEcounter.'].typo3ContentLanguage = "' . $this->contentTypo3Language . '";
-			RTEarea['.$RTEcounter.'].typo3ContentCharset = "' . $this->contentCharset . '";
-			RTEarea['.$RTEcounter.'].userUid = "' . $this->userUid . '";';
+			RTEarea[editornumber].sys_language_content = "' . $this->contentLanguageUid . '";
+			RTEarea[editornumber].typo3ContentLanguage = "' . $this->contentTypo3Language . '";
+			RTEarea[editornumber].typo3ContentCharset = "' . $this->contentCharset . '";
+			RTEarea[editornumber].userUid = "' . $this->userUid . '";';
 		}
 		
 			// Setting the plugin flags
 		$configureRTEInJavascriptString .= '
-			RTEarea['.$RTEcounter.'].plugin = new Object();
-			RTEarea['.$RTEcounter.'].pathToPluginDirectory = new Object();';
+			RTEarea[editornumber].plugin = new Object();
+			RTEarea[editornumber].pathToPluginDirectory = new Object();';
 		foreach ($this->pluginEnabledArray as $pluginId) {
 			$configureRTEInJavascriptString .= '
-			RTEarea['.$RTEcounter.'].plugin.'.$pluginId.' = true;';
+			RTEarea[editornumber].plugin.'.$pluginId.' = true;';
 			if (is_object($this->registeredPlugins[$pluginId])) {
 				$pathToPluginDirectory = $this->registeredPlugins[$pluginId]->getPathToPluginDirectory();
 				if ($pathToPluginDirectory) {
 					$configureRTEInJavascriptString .= '
-			RTEarea['.$RTEcounter.'].pathToPluginDirectory.'.$pluginId.' = "' . $pathToPluginDirectory . '";';
+			RTEarea[editornumber].pathToPluginDirectory.'.$pluginId.' = "' . $pathToPluginDirectory . '";';
 				}
 			}
 		}
 		
 			// Setting the buttons configuration
 		$configureRTEInJavascriptString .= '
-			RTEarea['.$RTEcounter.'].buttons = new Object();';
+			RTEarea[editornumber].buttons = new Object();';
 		if (is_array($this->thisConfig['buttons.'])) {
 			foreach ($this->thisConfig['buttons.'] as $buttonIndex => $conf) {
 				$button = substr($buttonIndex, 0, -1);
 				if (in_array($button,$this->toolbar)) {
 					$configureRTEInJavascriptString .= '
-			RTEarea['.$RTEcounter.'].buttons.'.$button.' = ' . $this->buildNestedJSArray($conf) . ';';
+			RTEarea[editornumber].buttons.'.$button.' = ' . $this->buildNestedJSArray($conf) . ';';
 				}
 			}
 		}
@@ -869,24 +929,24 @@ class tx_rtehtmlarea_base extends t3lib_rteapi {
 			// Setting the list of tags to be removed if specified in the RTE config
 		if (trim($this->thisConfig['removeTags']))  {
 			$configureRTEInJavascriptString .= '
-			RTEarea['.$RTEcounter.']["htmlRemoveTags"] = /^(' . implode('|', t3lib_div::trimExplode(',', $this->thisConfig['removeTags'], 1)) . ')$/i;';
+			RTEarea[editornumber]["htmlRemoveTags"] = /^(' . implode('|', t3lib_div::trimExplode(',', $this->thisConfig['removeTags'], 1)) . ')$/i;';
 		}
 		
 			// Setting the list of tags to be removed with their contents if specified in the RTE config
 		if (trim($this->thisConfig['removeTagsAndContents']))  {
 			$configureRTEInJavascriptString .= '
-			RTEarea['.$RTEcounter.']["htmlRemoveTagsAndContents"] = /^(' . implode('|', t3lib_div::trimExplode(',', $this->thisConfig['removeTagsAndContents'], 1)) . ')$/i;';
+			RTEarea[editornumber]["htmlRemoveTagsAndContents"] = /^(' . implode('|', t3lib_div::trimExplode(',', $this->thisConfig['removeTagsAndContents'], 1)) . ')$/i;';
 		}
 		
 			// Process default style configuration
 		$configureRTEInJavascriptString .= '
-			RTEarea['.$RTEcounter.'].defaultPageStyle = "' . $this->hostURL . $this->writeTemporaryFile('', 'defaultPageStyle', 'css', $this->buildStyleSheet()) . '";';
-			
+			RTEarea[editornumber].defaultPageStyle = "' . $this->hostURL . $this->writeTemporaryFile('', 'defaultPageStyle', 'css', $this->buildStyleSheet()) . '";';
+
 			// Setting the pageStyle
 		$filename = trim($this->thisConfig['contentCSS']) ? trim($this->thisConfig['contentCSS']) : 'EXT:' . $this->ID . '/res/contentcss/default.css';
 		$configureRTEInJavascriptString .= '
-			RTEarea['.$RTEcounter.'].pageStyle = "' . $this->getFullFileName($filename) .'";';
-		
+			RTEarea[editornumber].pageStyle = "' . $this->getFullFileName($filename) .'";';
+
 			// Process classes configuration
 		$classesConfigurationRequired = false;
 		foreach ($this->registeredPlugins as $pluginId => $plugin) {
@@ -901,13 +961,16 @@ class tx_rtehtmlarea_base extends t3lib_rteapi {
 			// Add Javascript configuration for registered plugins
 		foreach ($this->registeredPlugins as $pluginId => $plugin) {
 			if ($this->isPluginEnabled($pluginId)) {
-				$configureRTEInJavascriptString .= $plugin->buildJavascriptConfiguration($RTEcounter);
+				$configureRTEInJavascriptString .= $plugin->buildJavascriptConfiguration('editornumber');
 			}
 		}
-		
+			// Avoid premature reference to HTMLArea when being initially loaded by IRRE Ajax call
 		$configureRTEInJavascriptString .= '
-			RTEarea['.$RTEcounter.'].toolbar = '.$this->getJSToolbarArray().';
-			HTMLArea.initEditor('.$RTEcounter.');' . (!$this->is_FE() ? '' : '
+			RTEarea[editornumber].toolbar = '.$this->getJSToolbarArray().';
+			RTEarea.initEditor(editornumber);
+				}
+			};
+			configureEditorInstance["' . $textAreaId . '"]();'. (!$this->is_FE() ? '' : '
 			/*]]>*/');
 		return $configureRTEInJavascriptString;
 	}
@@ -984,16 +1047,16 @@ class tx_rtehtmlarea_base extends t3lib_rteapi {
 		$classesTagConvert = array( 'classesCharacter' => 'span', 'classesParagraph' => 'div', 'classesImage' => 'img', 'classesTable' => 'table', 'classesLinks' => 'a', 'classesTD' => 'td');
 		$classesTagArray = t3lib_div::trimExplode(',' , $classesTagList);
 		$configureRTEInJavascriptString = '
-			RTEarea['.$RTEcounter.']["classesTag"] = new Object();';
+			RTEarea[editornumber]["classesTag"] = new Object();';
 		foreach ($classesTagArray as $classesTagName) {
 			$HTMLAreaJSClasses = ($this->thisConfig[$classesTagName])?('"' . $this->cleanList($this->thisConfig[$classesTagName]) . '";'):'null;';
 			$configureRTEInJavascriptString .= '
-			RTEarea['.$RTEcounter.']["classesTag"]["'. $classesTagConvert[$classesTagName] .'"] = '. $HTMLAreaJSClasses;
+			RTEarea[editornumber]["classesTag"]["'. $classesTagConvert[$classesTagName] .'"] = '. $HTMLAreaJSClasses;
 		}
 		
 			// Include JS arrays of configured classes
 		$configureRTEInJavascriptString .= '
-			RTEarea['.$RTEcounter.']["classesUrl"] = "' . $this->hostURL . $this->writeTemporaryFile('', 'classes_'.$LANG->lang, 'js', $this->buildJSClassesArray()) . '";';
+			RTEarea[editornumber]["classesUrl"] = "' . $this->hostURL . $this->writeTemporaryFile('', 'classes_'.$LANG->lang, 'js', $this->buildJSClassesArray()) . '";';
 		
 		return $configureRTEInJavascriptString;
 	}
@@ -1389,17 +1452,23 @@ class tx_rtehtmlarea_base extends t3lib_rteapi {
 	 * @return	string		Javascript code
 	 */
 	function setSaveRTE($RTEcounter, $formName, $textareaId) {
-		return '
-		editornumber = '.$RTEcounter.';
-		if (RTEarea[editornumber]) {
-			document.'.$formName.'["'.$textareaId.'"].value = RTEarea[editornumber]["editor"].getHTML();
-		}
-		else {
-			OK=0;
-		}
-		';
+		return 'if (RTEarea[\'' . $textareaId . '\']) { document.' . $formName . '[\'' . $textareaId . '\'].value = RTEarea[\'' . $textareaId . '\'][\'editor\'].getHTML(); } else { OK = 0; };';
 	}
-	
+
+	/**
+	 * Return the Javascript code for copying the HTML code from the editor into the hidden input field.
+	 * This is for submit function of the form.
+	 *
+	 * @param	integer		$RTEcounter: The index number of the current RTE editing area within the form.
+	 * @param	string		$formName: the name of the form
+	 * @param	string		$textareaId: the id of the textarea
+	 *
+	 * @return	string		Javascript code
+	 */
+	function setDeleteRTE($RTEcounter, $formName, $textareaId) {
+		return 'if (RTEarea[\'' . $textareaId . '\']) { RTEarea[\'' . $textareaId . '\'].deleted = true;}';
+	}
+
 	/**
 	 * Return true if we are in the FE, but not in the FE editing feature of BE.
 	 *
@@ -1408,7 +1477,7 @@ class tx_rtehtmlarea_base extends t3lib_rteapi {
 	 
 	function is_FE() {
 		global $TSFE;
-		return is_object($TSFE) && !strstr($this->elementId,'TSFE_EDIT');
+		return is_object($TSFE) && is_array($this->LOCAL_LANG) && !strstr($this->elementId,'TSFE_EDIT');
 	}
 	
 	/**
