@@ -3,7 +3,7 @@
 *
 *  (c) 2002-2004, interactivetools.com, inc.
 *  (c) 2003-2004 dynarch.com
-*  (c) 2004-2008 Stanislas Rolland <stanislas.rolland(arobas)fructifor.ca>
+*  (c) 2004-2008 Stanislas Rolland <typo3(arobas)sjbr.ca>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -29,7 +29,7 @@
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/
 /*
- * TYPO3 SVN ID: $Id: htmlarea-gecko.js 3538 2008-04-06 19:07:42Z stan $
+ * TYPO3 SVN ID: $Id: htmlarea-gecko.js 4670 2009-01-08 21:10:33Z stan $
  */
 
 /***************************************************
@@ -66,7 +66,7 @@ HTMLArea.prototype._initEditMode = function () {
 			if (HTMLArea.is_wamcom) {
 				this._doc.open();
 				this._doc.close();
-				this._initIframeTimer = window.setTimeout("HTMLArea.initIframe(" + this._editorNumber + ");", 500);
+				this._initIframeTimer = window.setTimeout("HTMLArea.initIframe(\'" + this._editorNumber + "\');", 500);
 				return false;
 			}
 		}
@@ -318,7 +318,6 @@ HTMLArea.prototype._selectionEmpty = function(sel) {
  * handle DOM mutations when moving back to the bookmark.
  */
 HTMLArea.prototype.getBookmark = function (range) {
-		// For performance, includeNodes=true if intended to SelectBookmark.
 		// Create the bookmark info (random IDs).
 	var bookmark = {
 		startId : (new Date()).valueOf() + Math.floor(Math.random()*1000) + 'S',
@@ -382,11 +381,21 @@ HTMLArea.prototype.moveToBookmark = function (bookmark) {
 	var endSpan    = this.getBookmarkNode(bookmark, false);
 
 	var range = this._createRange();
-	range.setStartBefore(startSpan);
+		// If the previous sibling is a text node, let the anchor have it as parent
+	if (startSpan.previousSibling && startSpan.previousSibling.nodeType == 3) {
+		range.setStart(startSpan.previousSibling, startSpan.previousSibling.data.length);
+	} else {
+		range.setStartBefore(startSpan);
+	}
 	HTMLArea.removeFromParent(startSpan);
-		// If collapsed, the end span will not be available.
+		// If the bookmarked range was collapsed, the end span will not be available
 	if (endSpan) {
-		range.setEndBefore(endSpan);
+			// If the next sibling is a text node, let the anchor have it as parent
+		if (endSpan.nextSibling && endSpan.nextSibling.nodeType == 3) {
+			range.setEnd(endSpan.nextSibling, 0);
+		} else {
+			range.setEndBefore(endSpan);
+		}
 		HTMLArea.removeFromParent(endSpan);
 	} else {
 		range.collapse(true);
@@ -722,101 +731,103 @@ HTMLArea.prototype._checkInsertP = function() {
 HTMLArea.prototype._detectURL = function(ev) {
 	var editor = this;
 	var s = this._getSelection();
-	var autoWrap = function (textNode, tag) {
-		var rightText = textNode.nextSibling;
-		if (typeof(tag) == 'string') tag = editor._doc.createElement(tag);
-		var a = textNode.parentNode.insertBefore(tag, rightText);
-		HTMLArea.removeFromParent(textNode);
-		a.appendChild(textNode);
-		rightText.data += " ";
-		s.collapse(rightText, rightText.data.length);
-		HTMLArea._stopEvent(ev);
-
-		editor._unLink = function() {
-			var t = a.firstChild;
-			a.removeChild(t);
-			a.parentNode.insertBefore(t, a);
-			HTMLArea.removeFromParent(a);
-			t.parentNode.normalize();
-			editor._unLink = null;
-			editor._unlinkOnUndo = false;
+	if (this.getParentElement(s).nodeName.toLowerCase() != 'a') {
+		var autoWrap = function (textNode, tag) {
+			var rightText = textNode.nextSibling;
+			if (typeof(tag) == 'string') tag = editor._doc.createElement(tag);
+			var a = textNode.parentNode.insertBefore(tag, rightText);
+			HTMLArea.removeFromParent(textNode);
+			a.appendChild(textNode);
+			rightText.data += " ";
+			s.collapse(rightText, rightText.data.length);
+			HTMLArea._stopEvent(ev);
+	
+			editor._unLink = function() {
+				var t = a.firstChild;
+				a.removeChild(t);
+				a.parentNode.insertBefore(t, a);
+				HTMLArea.removeFromParent(a);
+				t.parentNode.normalize();
+				editor._unLink = null;
+				editor._unlinkOnUndo = false;
+			};
+	
+			editor._unlinkOnUndo = true;
+			return a;
 		};
-
-		editor._unlinkOnUndo = true;
-		return a;
-	};
-
-	switch(ev.which) {
-			// Space or Enter or >, see if the text just typed looks like a URL, or email address and link it accordingly
-		case 13:	// Enter
-			if(ev.shiftKey || editor.config.disableEnterParagraphs) break;
-				//Space
-		case 32:
-			if(s && s.isCollapsed && s.anchorNode.nodeType == 3 && s.anchorNode.data.length > 3 && s.anchorNode.data.indexOf('.') >= 0) {
-				var midStart = s.anchorNode.data.substring(0,s.anchorOffset).search(/[a-zA-Z0-9]+\S{3,}$/);
-				if(midStart == -1) break;
-				if(this._getFirstAncestor(s, 'a')) break; // already in an anchor
-				var matchData = s.anchorNode.data.substring(0,s.anchorOffset).replace(/^.*?(\S*)$/, '$1');
-				if (matchData.indexOf('@') != -1) {
-					var m = matchData.match(HTMLArea.RE_email);
+	
+		switch(ev.which) {
+				// Space or Enter or >, see if the text just typed looks like a URL, or email address and link it accordingly
+			case 13:	// Enter
+				if(ev.shiftKey || editor.config.disableEnterParagraphs) break;
+					//Space
+			case 32:
+				if(s && s.isCollapsed && s.anchorNode.nodeType == 3 && s.anchorNode.data.length > 3 && s.anchorNode.data.indexOf('.') >= 0) {
+					var midStart = s.anchorNode.data.substring(0,s.anchorOffset).search(/[a-zA-Z0-9]+\S{3,}$/);
+					if(midStart == -1) break;
+					if(this._getFirstAncestor(s, 'a')) break; // already in an anchor
+					var matchData = s.anchorNode.data.substring(0,s.anchorOffset).replace(/^.*?(\S*)$/, '$1');
+					if (matchData.indexOf('@') != -1) {
+						var m = matchData.match(HTMLArea.RE_email);
+						if(m) {
+							var leftText  = s.anchorNode;
+							var rightText = leftText.splitText(s.anchorOffset);
+							var midText   = leftText.splitText(midStart);
+							var midEnd = midText.data.search(/[^a-zA-Z0-9\.@_\-]/);
+							if (midEnd != -1) var endText = midText.splitText(midEnd);
+							autoWrap(midText, 'a').href = 'mailto:' + m[0];
+							break;
+						}
+					}
+					var m = matchData.match(HTMLArea.RE_url);
 					if(m) {
 						var leftText  = s.anchorNode;
 						var rightText = leftText.splitText(s.anchorOffset);
 						var midText   = leftText.splitText(midStart);
-						var midEnd = midText.data.search(/[^a-zA-Z0-9\.@_\-]/);
+						var midEnd = midText.data.search(/[^a-zA-Z0-9\._\-\/\&\?=:@]/);
 						if (midEnd != -1) var endText = midText.splitText(midEnd);
-						autoWrap(midText, 'a').href = 'mailto:' + m[0];
+						autoWrap(midText, 'a').href = (m[1] ? m[1] : 'http://') + m[2];
 						break;
 					}
 				}
-				var m = matchData.match(HTMLArea.RE_url);
-				if(m) {
-					var leftText  = s.anchorNode;
-					var rightText = leftText.splitText(s.anchorOffset);
-					var midText   = leftText.splitText(midStart);
-					var midEnd = midText.data.search(/[^a-zA-Z0-9\._\-\/\&\?=:@]/);
-					if (midEnd != -1) var endText = midText.splitText(midEnd);
-					autoWrap(midText, 'a').href = (m[1] ? m[1] : 'http://') + m[2];
-					break;
-				}
-			}
-			break;
-		default:
-			if(ev.keyCode == 27 || (editor._unlinkOnUndo && ev.ctrlKey && ev.which == 122) ) {
-				if(this._unLink) {
-					this._unLink();
-					HTMLArea._stopEvent(ev);
-				}
 				break;
-			} else if(ev.which || ev.keyCode == 8 || ev.keyCode == 46) {
-				this._unlinkOnUndo = false;
-				if(s.anchorNode && s.anchorNode.nodeType == 3) {
-						// See if we might be changing a link
-					var a = this._getFirstAncestor(s, 'a');
-					if(!a) break; // not an anchor
-					if(!a._updateAnchTimeout) {
-						if(s.anchorNode.data.match(HTMLArea.RE_email) && (a.href.match('mailto:' + s.anchorNode.data.trim()))) {
-							var textNode = s.anchorNode;
-							var fn = function() {
-								a.href = 'mailto:' + textNode.data.trim();
+			default:
+				if(ev.keyCode == 27 || (editor._unlinkOnUndo && ev.ctrlKey && ev.which == 122) ) {
+					if(this._unLink) {
+						this._unLink();
+						HTMLArea._stopEvent(ev);
+					}
+					break;
+				} else if(ev.which || ev.keyCode == 8 || ev.keyCode == 46) {
+					this._unlinkOnUndo = false;
+					if(s.anchorNode && s.anchorNode.nodeType == 3) {
+							// See if we might be changing a link
+						var a = this._getFirstAncestor(s, 'a');
+						if(!a) break; // not an anchor
+						if(!a._updateAnchTimeout) {
+							if(s.anchorNode.data.match(HTMLArea.RE_email) && (a.href.match('mailto:' + s.anchorNode.data.trim()))) {
+								var textNode = s.anchorNode;
+								var fn = function() {
+									a.href = 'mailto:' + textNode.data.trim();
+									a._updateAnchTimeout = setTimeout(fn, 250);
+								};
 								a._updateAnchTimeout = setTimeout(fn, 250);
-							};
-							a._updateAnchTimeout = setTimeout(fn, 250);
-							break;
-						}
-						var m = s.anchorNode.data.match(HTMLArea.RE_url);
-						if(m &&  a.href.match(s.anchorNode.data.trim())) {
-							var textNode = s.anchorNode;
-							var fn = function() {
-								var m = textNode.data.match(HTMLArea.RE_url);
-								a.href = (m[1] ? m[1] : 'http://') + m[2];
+								break;
+							}
+							var m = s.anchorNode.data.match(HTMLArea.RE_url);
+							if(m &&  a.href.match(s.anchorNode.data.trim())) {
+								var textNode = s.anchorNode;
+								var fn = function() {
+									var m = textNode.data.match(HTMLArea.RE_url);
+									a.href = (m[1] ? m[1] : 'http://') + m[2];
+									a._updateAnchTimeout = setTimeout(fn, 250);
+								}
 								a._updateAnchTimeout = setTimeout(fn, 250);
 							}
-							a._updateAnchTimeout = setTimeout(fn, 250);
 						}
 					}
 				}
-			}
-			break;
+				break;
+		}
 	}
 };
