@@ -31,7 +31,7 @@
 /*
  * Table Operations Plugin for TYPO3 htmlArea RTE
  *
- * TYPO3 SVN ID: $Id: table-operations.js 4102 2008-09-13 23:04:59Z stan $
+ * TYPO3 SVN ID: $Id: table-operations.js 4864 2009-01-26 16:30:48Z stan $
  */
 TableOperations = HTMLArea.Plugin.extend({
 		
@@ -1350,42 +1350,44 @@ TableOperations = HTMLArea.Plugin.extend({
 	 * @return	void
 	 */
 	buildSizeAndHeadersFieldset : function (doc, table, content, fieldsetClass) {
-		var fieldset = doc.createElement("fieldset");
-		if (fieldsetClass) fieldset.className = fieldsetClass;
-		if (!table) {
-			TableOperations.insertLegend(doc, fieldset, "Size and Headers");
-			TableOperations.buildInput(doc, fieldset, "f_rows", "Rows:", "Number of rows", "", "5", ((this.properties && this.properties.numberOfRows && this.properties.numberOfRows.defaultValue) ? this.properties.numberOfRows.defaultValue : "2"), "fr");
-			TableOperations.insertSpace(doc, fieldset);
-			TableOperations.buildInput(doc, fieldset, "f_cols", "Cols:", "Number of columns", "", "5", ((this.properties && this.properties.numberOfColumns && this.properties.numberOfColumns.defaultValue) ? this.properties.numberOfColumns.defaultValue : "4"), "fr");
-		} else {
-			TableOperations.insertLegend(doc, fieldset, "Headers");
-		}
-		if (this.removedProperties.indexOf("headers") == -1) {
-			var ul = doc.createElement("ul");
-			fieldset.appendChild(ul);
-			var li = doc.createElement("li");
-			ul.appendChild(li);
+		if (!table || this.removedProperties.indexOf("headers") == -1) {
+			var fieldset = doc.createElement("fieldset");
+			if (fieldsetClass) fieldset.className = fieldsetClass;
 			if (!table) {
-				var selected = (this.properties && this.properties.headers && this.properties.headers.defaultValue) ? this.properties.headers.defaultValue : "top";
+				TableOperations.insertLegend(doc, fieldset, "Size and Headers");
+				TableOperations.buildInput(doc, fieldset, "f_rows", "Rows:", "Number of rows", "", "5", ((this.properties && this.properties.numberOfRows && this.properties.numberOfRows.defaultValue) ? this.properties.numberOfRows.defaultValue : "2"), "fr");
+				TableOperations.insertSpace(doc, fieldset);
+				TableOperations.buildInput(doc, fieldset, "f_cols", "Cols:", "Number of columns", "", "5", ((this.properties && this.properties.numberOfColumns && this.properties.numberOfColumns.defaultValue) ? this.properties.numberOfColumns.defaultValue : "4"), "fr");
 			} else {
-				var selected = "none";
-				var thead = table.getElementsByTagName("thead");
-				var tbody = table.getElementsByTagName("tbody");
-				if (thead.length && thead[0].rows.length) {
-					selected = "top";
-				} else if (tbody.length && tbody[0].rows.length) {
-					if (HTMLArea._hasClass(tbody[0].rows[0], this.useHeaderClass)) {
-						selected = "both";
-					} else if (tbody[0].rows[0].cells.length && tbody[0].rows[0].cells[0].nodeName.toLowerCase() == "th") {
-						selected = "left";
+				TableOperations.insertLegend(doc, fieldset, "Headers");
+			}
+			if (this.removedProperties.indexOf("headers") == -1) {
+				var ul = doc.createElement("ul");
+				fieldset.appendChild(ul);
+				var li = doc.createElement("li");
+				ul.appendChild(li);
+				if (!table) {
+					var selected = (this.properties && this.properties.headers && this.properties.headers.defaultValue) ? this.properties.headers.defaultValue : "top";
+				} else {
+					var selected = "none";
+					var thead = table.getElementsByTagName("thead");
+					var tbody = table.getElementsByTagName("tbody");
+					if (thead.length && thead[0].rows.length) {
+						selected = "top";
+					} else if (tbody.length && tbody[0].rows.length) {
+						if (HTMLArea._hasClass(tbody[0].rows[0], this.useHeaderClass)) {
+							selected = "both";
+						} else if (tbody[0].rows[0].cells.length && tbody[0].rows[0].cells[0].nodeName.toLowerCase() == "th") {
+							selected = "left";
+						}
 					}
 				}
+				var selectHeaders = TableOperations.buildSelectField(doc, li, "f_headers", "Headers:", "fr", "floating", "Table headers", ["No header cells", "Header cells on top", "Header cells on left", "Header cells on top and left"], ["none", "top", "left", "both"],  new RegExp((selected ? selected : "top"), "i"));
+				this.removeOptions(selectHeaders, "headers");
 			}
-			var selectHeaders = TableOperations.buildSelectField(doc, li, "f_headers", "Headers:", "fr", "floating", "Table headers", ["No header cells", "Header cells on top", "Header cells on left", "Header cells on top and left"], ["none", "top", "left", "both"],  new RegExp((selected ? selected : "top"), "i"));
-			this.removeOptions(selectHeaders, "headers");
+			TableOperations.insertSpace(doc, fieldset);
+			content.appendChild(fieldset);
 		}
-		TableOperations.insertSpace(doc, fieldset);
-		content.appendChild(fieldset);
 	},
 	
 	buildLayoutFieldset : function(doc, el, content, fieldsetClass) {
@@ -1574,7 +1576,7 @@ TableOperations = HTMLArea.Plugin.extend({
 		(selected.match(/([^\s]*)\s/)) && (selected = RegExp.$1);
 		selectBorderStyle = TableOperations.buildSelectField(doc, fieldset, "f_st_borderStyle", "Border style:", "fr", "floating", "Border style", ["Not set", "No border", "Dotted", "Dashed", "Solid", "Double", "Groove", "Ridge", "Inset", "Outset"], ["not set", "none", "dotted", "dashed", "solid", "double", "groove", "ridge", "inset", "outset"], new RegExp((selected ? selected : "not set"), "i"));
 		selectBorderStyle.onchange = function() { setBorderFieldsVisibility(this.value == "none"); };
-		this.removeOptions(selectBorderStyle, "f_st_borderStyle");
+		this.removeOptions(selectBorderStyle, "borderStyle");
 		TableOperations.buildInput(doc, fieldset, "f_st_borderWidth", "Border width:", "Border width", "pixels", "5", f_st_borderWidth, "fr", "floating", "postlabel", borderFields);
 		TableOperations.insertSpace(doc, fieldset, borderFields);
 		
