@@ -3,7 +3,7 @@
 *  Copyright notice
 *
 *  (c) 1999-2008 Kasper Skaarhoj (kasperYYYY@typo3.com)
-*  (c) 2005-2008 Stanislas Rolland <stanislas.rolland(arobas)fructifor.ca>
+*  (c) 2005-2008 Stanislas Rolland <typo3(arobas)sjbr.ca>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -32,10 +32,10 @@
  *
  * Adapted for htmlArea RTE by Stanislas Rolland
  *
- * $Id: class.tx_rtehtmlarea_browse_links.php 3644 2008-05-08 03:38:31Z stan $
+ * $Id: class.tx_rtehtmlarea_browse_links.php 4152 2008-09-19 05:17:01Z stan $
  *
  * @author	Kasper Skaarhoj <kasperYYYY@typo3.com>
- * @author	Stanislas Rolland <stanislas.rolland(arobas)fructifor.ca>
+ * @author	Stanislas Rolland <typo3(arobas)sjbr.ca>
  */
 
 require_once (PATH_typo3.'class.browse_links.php');
@@ -209,12 +209,13 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 
 	var $editorNo;
 	var $buttonConfig = array();
-
+	
+	public $anchorTypes = array( 'page', 'url', 'file', 'mail', 'spec');
 	protected $classesAnchorDefault = array();
 	protected $classesAnchorDefaultTitle = array();
 	protected $classesAnchorDefaultTarget = array();
 	protected $classesAnchorJSOptions = array();
-	public $allowedItems;
+	public $allowedItems;    
 
 	/**
 	 * Constructor:
@@ -225,63 +226,20 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 	function init()	{
 		global $BE_USER,$BACK_PATH,$LANG,$TYPO3_CONF_VARS;
 
-			// Main GPvars:
-		$this->pointer = t3lib_div::_GP('pointer');
-		$this->bparams = t3lib_div::_GP('bparams');
-		$this->P = t3lib_div::_GP('P');
-		$this->RTEtsConfigParams = t3lib_div::_GP('RTEtsConfigParams');
-		$this->expandPage = t3lib_div::_GP('expandPage');
-		$this->expandFolder = t3lib_div::_GP('expandFolder');
-		$this->PM = t3lib_div::_GP('PM');
-		$this->contentTypo3Language = t3lib_div::_GP('contentTypo3Language');
-		$this->contentTypo3Charset = t3lib_div::_GP('contentTypo3Charset');
-		$this->editorNo = t3lib_div::_GP('editorNo');
-
-			// Find "mode"
-		$this->mode=t3lib_div::_GP('mode');
-		if (!$this->mode)	{
-			$this->mode='rte';
-		}
-
-			// init hook objects:
-		$this->hookObjects = array();
-		if (is_array($TYPO3_CONF_VARS['SC_OPTIONS']['typo3/class.browse_links.php']['browseLinksHook'])) {
-			foreach($TYPO3_CONF_VARS['SC_OPTIONS']['typo3/class.browse_links.php']['browseLinksHook'] as $classData) {
-				$processObject = &t3lib_div::getUserObj($classData);
-
-				if(!($processObject instanceof t3lib_browseLinksHook)) {
-					throw new UnexpectedValueException('$processObject must implement interface t3lib_browseLinksHook', 1195115652);
-				}
-
-				$parameters = array();
-				$processObject->init($this, $parameters);
-				$this->hookObjects[] = $processObject;
-			}
-		}
-
-			// Site URL
-		$this->siteURL = t3lib_div::getIndpEnv('TYPO3_SITE_URL');	// Current site url
-
-			// the script to link to
-		$this->thisScript = t3lib_div::getIndpEnv('SCRIPT_NAME');
+		$this->initVariables();
+		$this->initConfiguration();
+		$this->initHookObjects('ext/rtehtmlarea/mod3/class.tx_rtehtmlarea_browse_links.php');
 
 			// CurrentUrl - the current link url must be passed around if it exists
-		if ($this->mode=='wizard')	{
-			$currentLinkParts = t3lib_div::trimExplode(' ',$this->P['currentValue']);
-			$this->curUrlArray = array(
-				'target' => $currentLinkParts[1]
-			);
-			$this->curUrlInfo=$this->parseCurUrl($this->siteURL.'?id='.$currentLinkParts[0],$this->siteURL);
-		} else {
-			$this->curUrlArray = t3lib_div::_GP('curUrl');
-			if ($this->curUrlArray['all'])	{
-				$this->curUrlArray=t3lib_div::get_tag_attributes($this->curUrlArray['all']);
-			}
-			$this->curUrlInfo=$this->parseCurUrl($this->curUrlArray['href'],$this->siteURL);
+		$this->curUrlArray = t3lib_div::_GP('curUrl');
+		if ($this->curUrlArray['all'])	{
+			$this->curUrlArray=t3lib_div::get_tag_attributes($this->curUrlArray['all']);
 		}
+			// Note: parseCurUrl will invoke the hooks
+		$this->curUrlInfo = $this->parseCurUrl($this->curUrlArray['href'],$this->siteURL);
 
 			// Determine nature of current url:
-		$this->act=t3lib_div::_GP('act');
+		$this->act = t3lib_div::_GP('act');
 		if (!$this->act)	{
 			$this->act=$this->curUrlInfo['act'];
 		}
@@ -291,6 +249,7 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 
 			// Rich Text Editor specific configuration:
 		$addPassOnParams='';
+		$classSelected = array();
 		if ((string)$this->mode=='rte')	{
 			$RTEtsConfigParts = explode(':',$this->RTEtsConfigParams);
 			$addPassOnParams .= '&RTEtsConfigParams='.rawurlencode($this->RTEtsConfigParams);
@@ -308,7 +267,6 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 				} else {
 					$classesAnchorArray = t3lib_div::trimExplode(',',$this->thisConfig['classesLinks'], 1);
 				}
-				$anchorTypes = array( 'page', 'url', 'file', 'mail', 'spec');
 				$classesAnchor = array();
 				$classesAnchor['all'] = array();
 				if (is_array($RTEsetup['properties']['classesAnchor.'])) {
@@ -316,7 +274,7 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 					while(list($label,$conf)=each($RTEsetup['properties']['classesAnchor.'])) {
 						if (in_array($conf['class'], $classesAnchorArray)) {
 							$classesAnchor['all'][] = $conf['class'];
-							if (in_array($conf['type'], $anchorTypes)) {
+							if (in_array($conf['type'], $this->anchorTypes)) {
 								$classesAnchor[$conf['type']][] = $conf['class'];
 								if (is_array($this->thisConfig['classesAnchor.']) && is_array($this->thisConfig['classesAnchor.']['default.']) && $this->thisConfig['classesAnchor.']['default.'][$conf['type']] == $conf['class']) {
 									$this->classesAnchorDefault[$conf['type']] = $conf['class'];
@@ -331,15 +289,13 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 						}
 					}
 				}
-				reset($anchorTypes);
-				while (list(, $anchorType) = each($anchorTypes) ) {
-					reset($classesAnchorArray);
-					while(list(,$class)=each($classesAnchorArray)) {
+				foreach ($this->anchorTypes as $anchorType) {
+					foreach ($classesAnchorArray as $class) {
 						if (!in_array($class, $classesAnchor['all']) || (in_array($class, $classesAnchor['all']) && is_array($classesAnchor[$anchorType]) && in_array($class, $classesAnchor[$anchorType]))) {
 							$selected = '';
-							if ($this->setClass == $class) $selected = 'selected="selected"';
-							if (!$this->setClass && $this->classesAnchorDefault[$anchorType] == $class) {
+							if ($this->setClass == $class || (!$this->setClass && $this->classesAnchorDefault[$anchorType] == $class)) {
 								$selected = 'selected="selected"';
+								$classSelected[$anchorType] = true;
 							}
 							$classLabel = (is_array($RTEsetup['properties']['classes.']) && is_array($RTEsetup['properties']['classes.'][$class.'.']) && $RTEsetup['properties']['classes.'][$class.'.']['name']) ? $this->getPageConfigLabel($RTEsetup['properties']['classes.'][$class.'.']['name'], 0) : $class;
 							$classStyle = (is_array($RTEsetup['properties']['classes.']) && is_array($RTEsetup['properties']['classes.'][$class.'.']) && $RTEsetup['properties']['classes.'][$class.'.']['value']) ? $RTEsetup['properties']['classes.'][$class.'.']['value'] : '';
@@ -356,17 +312,147 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 		}
 
 			// Initializing the target value (RTE)
-		$this->setTarget = $this->curUrlArray['target'];
+			// Unset the target if it is set to a value different than default and if no class is selected and the target field is not displayed
+			// In other words, do not forward the target if we changed tab and the target field is not displayed
+		$this->setTarget = (isset($this->curUrlArray['target'])
+				&& !(
+					($this->curUrlArray['target'] != $this->thisConfig['defaultLinkTarget'])
+					&& !$classSelected[$this->act]
+					&& is_array($this->buttonConfig['targetSelector.']) && $this->buttonConfig['targetSelector.']['disabled'] && is_array($this->buttonConfig['popupSelector.']) && $this->buttonConfig['popupSelector.']['disabled'])
+				) ? $this->curUrlArray['target'] : '';
 		if ($this->thisConfig['defaultLinkTarget'] && !isset($this->curUrlArray['target']))	{
 			$this->setTarget=$this->thisConfig['defaultLinkTarget'];
 		}
 
 			// Creating backend template object:
 		$this->doc = t3lib_div::makeInstance('template');
-		$this->doc->bodyTagAdditions = 'onLoad="initDialog();"';
+		$this->doc->bodyTagAdditions = $this->getBodyTagAdditions();
 		$this->doc->docType= 'xhtml_trans';
 		$this->doc->backPath = $BACK_PATH;
 
+			// Load the Prototype library and browse_links.js
+		$this->doc->loadJavascriptLib('contrib/prototype/prototype.js');
+		$this->doc->loadJavascriptLib('js/browse_links.js');
+
+		$this->doc->getContextMenuCode();
+	}
+
+	/**
+	 * Initialize class variables
+	 *
+	 * @return	void
+	 */
+	public function initVariables() {
+		
+			// Process bparams
+		$this->bparams = t3lib_div::_GP('bparams');
+		$pArr = explode('|', $this->bparams);
+		$pRteArr = explode(':', $pArr[1]);
+		$this->editorNo = $pRteArr[0];
+		$this->contentTypo3Language = $pRteArr[1];
+		$this->contentTypo3Charset = $pRteArr[2];
+		$this->RTEtsConfigParams = $pArr[2];
+		if (!$this->editorNo) {
+			$this->editorNo = t3lib_div::_GP('editorNo');
+			$this->contentTypo3Language = t3lib_div::_GP('contentTypo3Language');
+			$this->contentTypo3Charset = t3lib_div::_GP('contentTypo3Charset');
+			$this->RTEtsConfigParams = t3lib_div::_GP('RTEtsConfigParams');
+		}
+		$this->pointer = t3lib_div::_GP('pointer');
+		$this->expandPage = t3lib_div::_GP('expandPage');
+		$this->expandFolder = t3lib_div::_GP('expandFolder');
+		$this->P = t3lib_div::_GP('P');
+		$this->PM = t3lib_div::_GP('PM');
+		$pArr[1] = implode(':', array($this->editorNo, $this->contentTypo3Language, $this->contentTypo3Charset));
+		$pArr[2] = $this->RTEtsConfigParams;
+		$this->bparams = implode('|', $pArr);
+		
+			// Find "mode"
+		$this->mode = t3lib_div::_GP('mode');
+		if (!$this->mode)	{
+			$this->mode = 'rte';
+		}
+			// Current site url
+		$this->siteURL = t3lib_div::getIndpEnv('TYPO3_SITE_URL');
+		
+			// the script to link to
+		$this->thisScript = t3lib_div::getIndpEnv('SCRIPT_NAME');
+		
+			// Init fileProcessor
+		$this->fileProcessor = t3lib_div::makeInstance('t3lib_basicFileFunctions');
+		$this->fileProcessor->init($GLOBALS['FILEMOUNTS'], $GLOBALS['TYPO3_CONF_VARS']['BE']['fileExtensions']);
+	}
+
+	/**
+	 * Initializes the configuration variables
+	 *
+	 * @return	void
+	 */
+	 public function initConfiguration() {
+		$this->thisConfig = $this->getRTEConfig();
+		$this->buttonConfig = $this->getButtonConfig('link');
+	 }
+
+	/**
+	 * Get the RTE configuration from Page TSConfig
+	 *
+	 * @return	array		RTE configuration array
+	 */
+	protected function getRTEConfig()	{
+		global $BE_USER;
+		
+		$RTEtsConfigParts = explode(':', $this->RTEtsConfigParams);
+		$RTEsetup = $BE_USER->getTSConfig('RTE',t3lib_BEfunc::getPagesTSconfig($RTEtsConfigParts[5]));
+		$this->RTEProperties = $RTEsetup['properties'];
+		return t3lib_BEfunc::RTEsetup($this->RTEProperties, $RTEtsConfigParts[0],$RTEtsConfigParts[2],$RTEtsConfigParts[4]);
+	}
+
+	/**
+	 * Get the configuration of the button
+	 *
+	 * @param	string		$buttonName: the name of the button
+	 * @return	array		the configuration array of the image button
+	 */
+	protected function getButtonConfig($buttonName)	{
+		return ((is_array($this->thisConfig['buttons.']) && is_array($this->thisConfig['buttons.'][$buttonName.'.'])) ? $this->thisConfig['buttons.'][$buttonName.'.'] : array());
+	}
+
+	/**
+	 * Initialize hook objects implementing interface t3lib_browseLinksHook
+	 * @param	string		$hookKey: the hook key
+	 * @return	void
+	 */
+	protected function initHookObjects($hookKey) {
+		global $TYPO3_CONF_VARS;
+		if (is_array($TYPO3_CONF_VARS['SC_OPTIONS'][$hookKey]['browseLinksHook'])) {
+			foreach ($TYPO3_CONF_VARS['SC_OPTIONS'][$hookKey]['browseLinksHook'] as $classData) {
+				$processObject = &t3lib_div::getUserObj($classData);
+				if(!($processObject instanceof t3lib_browseLinksHook)) {
+					throw new UnexpectedValueException('$processObject must implement interface t3lib_browseLinksHook', 1195115652);
+				}
+				$parameters = array();
+				$processObject->init($this, $parameters);
+				$this->hookObjects[] = $processObject;
+			}
+		}
+	}
+
+	/**
+	 * Provide the additional parameters to be included in the template body tag
+	 *
+	 * @return	string		the body tag additions
+	 */
+	public function getBodyTagAdditions() {
+		return 'onLoad="initDialog();"';
+	}
+	
+	/**
+	 * Generate JS code to be used on the link insert/modify dialogue
+	 *
+	 * @return	string		the generated JS code
+	 */
+	function getJSCode()	{
+		global $BACK_PATH;
 			// BEGIN accumulation of header JavaScript:
 		$JScode = '';
 		$JScode.= '
@@ -375,6 +461,9 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 			var HTMLArea = window.opener.HTMLArea;
 
 			function initDialog() {
+				window.dialog = window.opener.HTMLArea.Dialog.TYPO3Link;
+				window.plugin = dialog.plugin;
+				window.HTMLArea = window.opener.HTMLArea;
 				dialog.captureEvents("skipUnload");
 			}
 				// This JavaScript is primarily for RTE/Link. jumpToUrl is used in the other cases as well...
@@ -389,19 +478,19 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 			var cur_class="'.($this->setClass?$this->setClass:'').'";
 			var cur_title="'.($this->setTitle?$this->setTitle:'').'";
 
-			function setTarget(value)	{
+			function browse_links_setTarget(value)	{
 				cur_target=value;
 				add_target="&curUrl[target]="+encodeURIComponent(value);
 			}
-			function setClass(value)	{
+			function browse_links_setClass(value)	{
 				cur_class=value;
 				add_class="&curUrl[class]="+encodeURIComponent(value);
 			}
-			function setTitle(value)	{
+			function browse_links_setTitle(value)	{
 				cur_title=value;
 				add_title="&curUrl[title]="+encodeURIComponent(value);
 			}
-			function setValue(value)	{
+			function browse_links_setHref(value)	{
 				cur_href=value;
 				add_href="&curUrl[href]="+value;
 			}
@@ -436,10 +525,10 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 				}
 				function link_current()	{	//
 					if (cur_href!="http://" && cur_href!="mailto:")	{
-						var setValue = cur_href+" "+cur_target+" "+cur_class+" "+cur_title;
-						if (setValue.substr(0,7)=="http://")	setValue = setValue.substr(7);
-						if (setValue.substr(0,7)=="mailto:")	setValue = setValue.substr(7);
-						updateValueInMainForm(setValue);
+						var browse_links_setHref = cur_href+" "+cur_target+" "+cur_class+" "+cur_title;
+						if (browse_links_setHref.substr(0,7)=="http://")	browse_links_setHref = browse_links_setHref.substr(7);
+						if (browse_links_setHref.substr(0,7)=="mailto:")	browse_links_setHref = browse_links_setHref.substr(7);
+						updateValueInMainForm(browse_links_setHref);
 						close();
 					}
 					return false;
@@ -463,27 +552,31 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 			$JScode.='
 				function link_typo3Page(id,anchor)	{
 					var theLink = \''.$this->siteURL.'?id=\'+id+(anchor?anchor:"");
-					if (document.ltargetform.anchor_title) setTitle(document.ltargetform.anchor_title.value);
-					if (document.ltargetform.anchor_class) setClass(document.ltargetform.anchor_class.value);
+					if (document.ltargetform.anchor_title) browse_links_setTitle(document.ltargetform.anchor_title.value);
+					if (document.ltargetform.anchor_class) browse_links_setClass(document.ltargetform.anchor_class.value);
+					if (document.ltargetform.ltarget) browse_links_setTarget(document.ltargetform.ltarget.value);
 					plugin.createLink(theLink,cur_target,cur_class,cur_title);
 					return false;
 				}
 				function link_folder(folder)	{	//
 					var theLink = \''.$this->siteURL.'\'+folder;
-					if (document.ltargetform.anchor_title) setTitle(document.ltargetform.anchor_title.value);
-					if (document.ltargetform.anchor_class) setClass(document.ltargetform.anchor_class.value);
+					if (document.ltargetform.anchor_title) browse_links_setTitle(document.ltargetform.anchor_title.value);
+					if (document.ltargetform.anchor_class) browse_links_setClass(document.ltargetform.anchor_class.value);
+					if (document.ltargetform.ltarget) browse_links_setTarget(document.ltargetform.ltarget.value);
 					plugin.createLink(theLink,cur_target,cur_class,cur_title);
 					return false;
 				}
 				function link_spec(theLink)	{	//
-					if (document.ltargetform.anchor_title) setTitle(document.ltargetform.anchor_title.value);
-					if (document.ltargetform.anchor_class) setClass(document.ltargetform.anchor_class.value);
+					if (document.ltargetform.anchor_title) browse_links_setTitle(document.ltargetform.anchor_title.value);
+					if (document.ltargetform.anchor_class) browse_links_setClass(document.ltargetform.anchor_class.value);
+					if (document.ltargetform.ltarget) browse_links_setTarget(document.ltargetform.ltarget.value);
 					plugin.createLink(theLink,cur_target,cur_class,cur_title);
 					return false;
 				}
 				function link_current()	{	//
-					if (document.ltargetform.anchor_title) setTitle(document.ltargetform.anchor_title.value);
-					if (document.ltargetform.anchor_class) setClass(document.ltargetform.anchor_class.value);
+					if (document.ltargetform.anchor_title) browse_links_setTitle(document.ltargetform.anchor_title.value);
+					if (document.ltargetform.anchor_class) browse_links_setClass(document.ltargetform.anchor_class.value);
+					if (document.ltargetform.ltarget) browse_links_setTarget(document.ltargetform.ltarget.value);
 					if (cur_href!="http://" && cur_href!="mailto:")	{
 						plugin.createLink(cur_href,cur_target,cur_class,cur_title);
 					}
@@ -562,24 +655,7 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 				}
 			}
 		';
-
-			// Finally, add the accumulated JavaScript to the template object:
-		$this->doc->JScode = $this->doc->wrapScriptTags($JScode);
-
-			// Debugging:
-		if (FALSE) debug(array(
-			'pointer' => $this->pointer,
-			'act' => $this->act,
-			'mode' => $this->mode,
-			'curUrlInfo' => $this->curUrlInfo,
-			'curUrlArray' => $this->curUrlArray,
-			'P' => $this->P,
-			'bparams' => $this->bparams,
-			'RTEtsConfigParams' => $this->RTEtsConfigParams,
-			'expandPage' => $this->expandPage,
-			'expandFolder' => $this->expandFolder,
-			'PM' => $this->PM,
-		),'Internal variables of Script Class:');
+		return $JScode;
 	}
 
 	/******************************************************************
@@ -680,7 +756,7 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 							<tr>
 								<td>'.$LANG->getLL('emailAddress',1).':</td>
 								<td><input type="text" name="lemail"'.$this->doc->formWidth(20).' value="'.htmlspecialchars($this->curUrlInfo['act']=='mail'?$this->curUrlInfo['info']:'').'" /> '.
-									'<input type="submit" value="'.$LANG->getLL('setLink',1).'" onclick="setTarget(\'\');setValue(\'mailto:\'+document.lurlform.lemail.value); return link_current();" /></td>
+									'<input type="submit" value="'.$LANG->getLL('setLink',1).'" onclick="browse_links_setTarget(\'\');browse_links_setHref(\'mailto:\'+document.lurlform.lemail.value); return link_current();" /></td>
 							</tr>
 						</table>
 					</form>';
@@ -697,7 +773,7 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 							<tr>
 								<td>URL:</td>
 								<td><input type="text" name="lurl"'.$this->doc->formWidth(20).' value="'.htmlspecialchars($this->curUrlInfo['act']=='url'?$this->curUrlInfo['info']:'http://').'" /> '.
-									'<input type="submit" value="'.$LANG->getLL('setLink',1).'" onclick="setValue(document.lurlform.lurl.value); return link_current();" /></td>
+									'<input type="submit" value="'.$LANG->getLL('setLink',1).'" onclick="browse_links_setHref(document.lurlform.lurl.value); return link_current();" /></td>
 							</tr>
 						</table>
 					</form>';
@@ -706,7 +782,7 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 			break;
 			case 'file':
 				$content.=$this->addAttributesForm();
-
+				
 				$foldertree = t3lib_div::makeInstance('tx_rtehtmlarea_folderTree');
 				$tree=$foldertree->getBrowsableTree();
 
@@ -724,13 +800,11 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 
 					// Create upload/create folder forms, if a path is given:
 				if ($BE_USER->getTSConfigVal('options.uploadFieldsInTopOfEB')) {
-					$fileProcessor = t3lib_div::makeInstance('t3lib_basicFileFunctions');
-					$fileProcessor->init($GLOBALS['FILEMOUNTS'], $GLOBALS['TYPO3_CONF_VARS']['BE']['fileExtensions']);
 					$path=$this->expandFolder;
 					if (!$path || !@is_dir($path))	{
-						$path = $fileProcessor->findTempFolder().'/';	// The closest TEMP-path is found
+						$path = $this->fileProcessor->findTempFolder().'/';	// The closest TEMP-path is found
 					}
-					if ($path!='/' && @is_dir($path))	{
+					if ($path!='/' && @is_dir($path)) { 
 						$uploadForm=$this->uploadForm($path);
 						$createFolder=$this->createFolder($path);
 					} else {
@@ -743,6 +817,8 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 					}
 				}
 
+				
+				
 				$content.= '
 			<!--
 			Wrapper table for folder tree / file list:
@@ -776,7 +852,7 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 
 								// URL + onclick event:
 							$onClickEvent='';
-							if (isset($v[$k2i.'.']['target']))	$onClickEvent.="setTarget('".$v[$k2i.'.']['target']."');";
+							if (isset($v[$k2i.'.']['target']))	$onClickEvent.="browse_links_setTarget('".$v[$k2i.'.']['target']."');";
 							$v[$k2i.'.']['url'] = str_replace('###_URL###',$this->siteURL,$v[$k2i.'.']['url']);
 							if (substr($v[$k2i.'.']['url'],0,7)=="http://" || substr($v[$k2i.'.']['url'],0,7)=='mailto:')	{
 								$onClickEvent.="cur_href=unescape('".rawurlencode($v[$k2i.'.']['url'])."');link_current();";
@@ -843,6 +919,8 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 
 			// End page, return content:
 		$content.= $this->doc->endPage();
+		$this->doc->JScodeArray['rtehtmlarea'] = $this->getJSCode();
+		$content = $this->doc->insertStylesAndJS($content);
 		return $content;
 	}
 
@@ -868,7 +946,7 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 			-->
 				<form action="" name="ltargetform" id="ltargetform">
 					<table border="0" cellpadding="2" cellspacing="1" id="typo3-linkTarget">'. $string;
-		if ((($this->act == 'page' && $this->curUrlInfo['act']=='page') || ($this->act == 'file' && $this->curUrlInfo['act']=='file') || ($this->act == 'url' && $this->curUrlInfo['act']!='page')) && $this->curUrlArray['href']) {
+		if ($this->act == $this->curUrlInfo['act'] && $this->act != 'mail' && $this->curUrlArray['href']) {
 			$form .='
 						<tr>
 							<td>
@@ -910,32 +988,28 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 
 		$ltarget = '';
 		if ($this->act != 'mail')	{
-			if (!($targetSelectorConfig['disabled'] && $popupSelectorConfig['disabled'])) {
-				$ltarget .= '
-						<tr>
-							<td>'.$LANG->getLL('target',1).':</td>
-							<td><input type="text" name="ltarget" onchange="setTarget(this.value);" value="'.htmlspecialchars($this->setTarget?$this->setTarget:(($this->setClass || !$this->classesAnchorDefault[$this->act])?'':$this->classesAnchorDefaultTarget[$this->act])).'"'.$this->doc->formWidth(10).' /></td>';
-				$ltarget .= '
-							<td colspan="2">';
-				if (!$targetSelectorConfig['disabled']) {
-					$ltarget .= '
-								<select name="ltarget_type" onchange="setTarget(this.options[this.selectedIndex].value);document.ltargetform.ltarget.value=this.options[this.selectedIndex].value;this.selectedIndex=0;">
-									<option></option>
-									<option value="_top">'.$LANG->getLL('top',1).'</option>
-									<option value="_blank">'.$LANG->getLL('newWindow',1).'</option>
-								</select>';
-				}
-				$ltarget .= '
-							</td>';
-			}
-
 			$ltarget .= '
-						</tr>';
+					<tr id="ltargetrow"'. (($targetSelectorConfig['disabled'] && $popupSelectorConfig['disabled']) ? ' style="display: none;"' : '') . '>
+						<td>'.$LANG->getLL('target',1).':</td>
+						<td><input type="text" name="ltarget" onchange="browse_links_setTarget(this.value);" value="'.htmlspecialchars($this->setTarget?$this->setTarget:(($this->setClass || !$this->classesAnchorDefault[$this->act])?'':$this->classesAnchorDefaultTarget[$this->act])).'"'.$this->doc->formWidth(10).' /></td>';
+			$ltarget .= '
+						<td colspan="2">';
+			if (!$targetSelectorConfig['disabled']) {
+				$ltarget .= '
+							<select name="ltarget_type" onchange="browse_links_setTarget(this.options[this.selectedIndex].value);document.ltargetform.ltarget.value=this.options[this.selectedIndex].value;this.selectedIndex=0;">
+								<option></option>
+								<option value="_top">'.$LANG->getLL('top',1).'</option>
+								<option value="_blank">'.$LANG->getLL('newWindow',1).'</option>
+							</select>';
+			}
+			$ltarget .= '
+						</td>
+					</tr>';
 			if (!$popupSelectorConfig['disabled']) {
 
 				$selectJS = 'if (document.ltargetform.popup_width.options[document.ltargetform.popup_width.selectedIndex].value>0 && document.ltargetform.popup_height.options[document.ltargetform.popup_height.selectedIndex].value>0)	{
 					document.ltargetform.ltarget.value = document.ltargetform.popup_width.options[document.ltargetform.popup_width.selectedIndex].value+\'x\'+document.ltargetform.popup_height.options[document.ltargetform.popup_height.selectedIndex].value;
-					setTarget(document.ltargetform.ltarget.value);
+					browse_links_setTarget(document.ltargetform.ltarget.value);
 					document.ltargetform.popup_width.selectedIndex=0;
 					document.ltargetform.popup_height.selectedIndex=0;
 				}';
@@ -981,18 +1055,25 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 							for (var i = HTMLArea.classesAnchorSetup.length; --i >= 0;) {
 								var anchorClass = HTMLArea.classesAnchorSetup[i];
 								if (anchorClass[\'name\'] == document.ltargetform.anchor_class.value) {
-									if (anchorClass[\'titleText\'] && document.ltargetform.anchor_title) document.ltargetform.anchor_title.value = anchorClass[\'titleText\'];
+									if (anchorClass[\'titleText\'] && document.ltargetform.anchor_title) {
+										document.ltargetform.anchor_title.value = anchorClass[\'titleText\'];
+										browse_links_setTitle(anchorClass[\'titleText\']);
+									}
 									if (anchorClass[\'target\']) {
 										if (document.ltargetform.ltarget) {
 											document.ltargetform.ltarget.value = anchorClass[\'target\'];
 										}
-										setTarget(anchorClass[\'target\']);
+										browse_links_setTarget(anchorClass[\'target\']);
+									} else if (document.ltargetform.ltarget && document.getElementById(\'ltargetrow\').style.display == \'none\') {
+											// Reset target to default if field is not displayed and class has no configured target
+										document.ltargetform.ltarget.value = \''. ($this->thisConfig['defaultLinkTarget']?$this->thisConfig['defaultLinkTarget']:'') .'\';
+										browse_links_setTarget(document.ltargetform.ltarget.value);
 									}
 									break;
 								}
 							}
 						}
-						setClass(document.ltargetform.anchor_class.value);
+						browse_links_setClass(document.ltargetform.anchor_class.value);
 					}
 				';
 			$selectClass ='
@@ -1031,6 +1112,8 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 		global $BACK_PATH;
 		$count=3;
 
+		if ($this->isReadOnlyFolder($path)) return '';
+
 			// Create header, showing upload path:
 		$header = t3lib_div::isFirstPartOfStr($path,PATH_site)?substr($path,strlen(PATH_site)):$path;
 		$code=$this->barheader($GLOBALS['LANG']->getLL('uploadImage').':');
@@ -1083,6 +1166,9 @@ class tx_rtehtmlarea_browse_links extends browse_links {
 	 */
 	function createFolder($path)	{
 		global $BACK_PATH;
+		
+		if ($this->isReadOnlyFolder($path)) return '';
+		
 			// Create header, showing upload path:
 		$header = t3lib_div::isFirstPartOfStr($path,PATH_site)?substr($path,strlen(PATH_site)):$path;
 		$code=$this->barheader($GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:file_newfolder.php.pagetitle').':');
@@ -1166,4 +1252,4 @@ if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/rtehtml
 	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/rtehtmlarea/mod3/class.tx_rtehtmlarea_browse_links.php']);
 }
 
-?>
\ No newline at end of file
+?>
